<?php

namespace SyncSpider\APIextension\EntityManager;

use Magento\Framework\{
    Api\ExtensibleDataInterface,
    Api\ExtensionAttributesFactory,
    Data\Collection,
    Model\AbstractModel
};
use Magento\Framework\EntityManager\Operation\ExtensionPool;
use Exception;

class ExtensionAttributesResolver
{
    public const IS_OBJECT_NEW_FLAG = 'monobunt_is_object_new';

    public const IS_ENTITY_MANAGER_ENTITY = 'monobunt_is_entity_manager_entity';

    /**
     * @var ExtensionAttributesFactory
     */
    private $extensionAttributesFactory;

    /**
     * @var ExtensionPool
     */
    private $extensionPool;

    public function __construct(ExtensionAttributesFactory $extensionAttributesFactory, ExtensionPool $extensionPool)
    {
        $this->extensionAttributesFactory = $extensionAttributesFactory;
        $this->extensionPool = $extensionPool;
    }

    /**
     * Get extension attributes factory
     *
     * @return ExtensionAttributesFactory
     */
    public function getExtensionAttributesFactory(): ExtensionAttributesFactory
    {
        return $this->extensionAttributesFactory;
    }

    /**
     * Get extensible interface name for given extensible entity
     *
     * @param ExtensibleDataInterface $entity
     *
     * @return string|null
     */
    public function getExtensibleInterfaceName(ExtensibleDataInterface $entity): string
    {
        return $this->extensionAttributesFactory->getExtensibleInterfaceName(get_class($entity));
    }

    /**
     * Whether given entity has extensible interface
     *
     * @param ExtensibleDataInterface $entity
     *
     * @return bool
     */
    public function hasExtensibleInterface($entity): bool
    {
        try {
            return $entity instanceof ExtensibleDataInterface && !!$this->getExtensibleInterfaceName($entity);
        } catch (Exception $e) {
            // suppress exceptions
        }

        return false;
    }

    /**
     * Get action flag for given action name
     *
     * @param string $actionName
     *
     * @return string
     */
    public function getActionFlag(string $actionName): string
    {
        return ExtensionPool::class . '_' . $actionName;
    }

    /**
     * Whether extension action flag exists on given entity
     *
     * @param AbstractModel|Collection $entity
     * @param string $actionName
     *
     * @return bool
     */
    public function hasActionFlag($entity, string $actionName): bool
    {
        if ($entity instanceof Collection) {
            return $entity->hasFlag($this->getActionFlag($actionName));
        }

        return $entity instanceof AbstractModel
            && $entity->getData($this->getActionFlag($actionName)) === true;
    }

    /**
     * Set extension action flag on given entity
     *
     * @param AbstractModel $entity
     * @param string $actionName
     *
     * @return bool
     */
    public function setActionFlag($entity, string $actionName)
    {
        if ($entity instanceof Collection) {
            return true && $entity->setFlag($this->getActionFlag($actionName));
        }

        return $entity instanceof AbstractModel
            && $entity->setData($this->getActionFlag($actionName), true);
    }

    /**
     * Remove extension action flag on given entity
     *
     * @param AbstractModel $entity
     * @param string $actionName
     *
     * @return bool
     */
    public function removeActionFlag($entity, string $actionName)
    {
        if ($entity instanceof Collection) {
            // collecitons do not support removal of flags so each action can be executed only once
            return false;
        }

        return $entity instanceof AbstractModel
            && $entity->setData($this->getActionFlag($actionName), null);
    }

    /**
     *  Whether current action is an update
     *
     * @param AbstractModel $entity
     * @param string $actionName
     *
     * @return bool
     */
    public function isUpdateAction($entity, string $actionName): bool
    {
        if ($entity instanceof Collection) {
            // collections do not have an update action
            return false;
        }

        return $entity instanceof AbstractModel
            && ($actionName === 'save' || $actionName === 'create')
            && ! $entity->isObjectNew()
            && ! $this->hasActionFlag($entity, static::IS_OBJECT_NEW_FLAG);
    }

    /**
     * Trigger entity manager extension actions for given entity, entity type and action
     *
     * @param AbstractModel $entity
     * @param string $entityType
     * @param string $actionName
     * @param string $suffix
     *
     * @return ExtensionAttributesResolver
     */
    public function triggerEntityActions($entity, string $entityType, string $actionName, string $suffix = '')
    {
        if ($this->isUpdateAction($entity, $actionName)) {
            $actionName = 'update';
        }

        $actionName .= $suffix;

        if (! $this->hasActionFlag($entity, $actionName)) {
            $this->setActionFlag($entity, $actionName);
            $actions = $this->extensionPool->getActions($entityType, $actionName);

            foreach ($actions as $action) {
                $action->execute($entity, ['actionName' => $actionName, 'entityType' => $entityType]);
            }

            $this->removeActionFlag($entity, $actionName);
        }

        return $this;
    }
}
