<?php

namespace SyncSpider\APIextension\EntityManager;

use Magento\Framework\DB\Select;
use Magento\Framework\EntityManager\Operation\ExtensionInterface;
use Magento\Sales\Api\Data\{OrderInterface, OrderExtensionInterface};
use SyncSpider\APIextension\Api\Data\SynchronizationDataInterface;
use Magento\Framework\App\ResourceConnection;
use Zend_Db_Expr;
use Exception;

class SyncspiderSynchronizationDataHandler implements ExtensionInterface
{
    /**
     * @var ResourceConnection
     */
    private $resourceConnection;

    /**
     * SyncspiderSynchronizationDataHandler constructor.
     *
     * @param ResourceConnection $resourceConnection
     */
    public function __construct(ResourceConnection $resourceConnection)
    {
        $this->resourceConnection = $resourceConnection;
    }

    /**
     * Assign extension attribute to order entity
     *
     * Assign "order.syncspider_read_date" column value as "order.extension_attributes.syncspider_read_date"
     * extension attribute
     *
     * @param OrderItemInterface $entity
     * @param array $arguments
     *
     * @return OrderItemInterface
     */
    public function execute($entity, $arguments = [])
    {
        if ($entity instanceof SynchronizationDataInterface && ! empty($arguments['entityIds'])) {
            // update "sales_order" table
            $this->updateTable(
                'sales_order',
                [
                    'syncspider_synchronization_id' => new Zend_Db_Expr(':syncspider_synchronization_id'),
                    'syncspider_read_date' => new Zend_Db_Expr(':syncspider_synchronization_timestamp')
                ],
                $arguments['entityIds'],
                [
                    'syncspider_synchronization_id' => $entity->getExecutionId(),
                    'syncspider_synchronization_timestamp' => $entity->getTimestamp()
                ]
            );
            // update "sales_order_grid" table
            $this->updateTable(
                'sales_order_grid',
                [
                    'syncspider_read_date' => new Zend_Db_Expr(':syncspider_synchronization_timestamp')
                ],
                $arguments['entityIds'],
                [
                    'syncspider_synchronization_timestamp' => $entity->getTimestamp()
                ]
            );
        }

        return $entity;
    }

    /**
     * Bind values to given select
     *
     * @param Select $select
     * @param array $values
     * @param string $prefix
     *
     * @return string[]
     */
    public function bindValues(Select $select, array $values, string $prefix)
    {
        $binds = [];
        $bindNames = [];

        foreach ($values as $key => $value) {
            $bindName = $prefix . '_' . $key;
            $bindNames[] = ':' . $bindName;
            $binds[$bindName] = $value;
        }

        $select->bind(array_merge(
            $select->getBind(),
            $binds
        ));
        return $bindNames;
    }

    /**
     * Update "sales_order" related table based on given data
     *
     * The related table is joined on "o.entity_id = t.entity_id" condition and can be used for "sales_order"
     * and "sales_order_gird" tables.
     *
     * @param string $tableName
     * @param array $values
     * @param string[] $entityIds
     * @param string $binds
     *
     * @return SyncspiderSynchronizationDataHandler
     */
    private function updateTable(
        string $tableName,
        array $values,
        array $entityIds,
        array $binds = []
    ): SyncspiderSynchronizationDataHandler {
        try {
            if (! empty($entityIds)) {
                $connection = $this->resourceConnection->getConnection();
                $select = $connection->select()
                    ->join(
                        ['o' => $this->resourceConnection->getTableName('sales_order')],
                        'o.entity_id = t.entity_id',
                        $values
                    );
                // attach main condition
                $bindNames = $this->bindValues($select, $entityIds, 'entity');
                $select->where('t.entity_id IN(' . implode(',', $bindNames) . ')');
                // create and execute update query
                $updateQuery = $connection->updateFromSelect($select, ['t' => $this->resourceConnection->getTableName($tableName)]);
                $connection->query(
                    $updateQuery,
                    array_merge($binds, $select->getBind())
                );
            }
        } catch (Exception $e) {
            // suppress exceptions
        }

        return $this;
    }
}
