<?php

namespace SyncSpider\APIextension\Helper;

use Magento\Framework\App\Helper\AbstractHelper;
use Magento\Framework\App\Helper\Context;
use Magento\Framework\Event\Observer;
use Magento\Framework\Model\AbstractModel;
use Magento\Framework\Module\Dir\Reader;
use Magento\Framework\App\ProductMetadataInterface;
use Magento\Framework\Module\ModuleListInterface;
use Magento\Framework\Filesystem\Driver\File;
use Magento\Store\Model\Store;
use Magento\Store\Model\StoreManagerInterface;
use Psr\Log\LoggerInterface;
use Exception;

class Data extends AbstractHelper
{
    public const MODULE_CODE = 'SyncSpider_APIextension';

    public const PACKAGE_NAME = 'syncspider/apiextension';

    /**
     * @var StoreManagerInterface
     */
    private $storeManager;

    /**
     * @var Store
     */
    private $store;

    /**
     * @var ProductMetadataInterface
     */
    private $productMetadata;

    /**
     * @var ModuleListInterface
     */
    private $moduleList;

    /**
     * @var Reader
     */
    private $moduleReader;

    /**
     * @var File
     */
    private $fileDriver;

    /**
     * @var LoggerInterface
     */
    private $logger;

    /**
     * @param Context $context
     * @param StoreManagerInterface $storeManager
     * @param ProductMetadataInterface $productMetadata
     * @param ModuleListInterface $moduleList
     * @param Reader $moduleReader
     * @param File $fileDriver
     */
    public function __construct(
        Context $context,
        StoreManagerInterface $storeManager,
        ProductMetadataInterface $productMetadata,
        ModuleListInterface $moduleList,
        Reader $moduleReader,
        File $fileDriver,
        LoggerInterface $logger
    ) {
        parent::__construct($context);
        $this->storeManager = $storeManager;
        $this->store = $this->storeManager->getStore();
        $this->productMetadata = $productMetadata;
        $this->moduleList = $moduleList;
        $this->moduleReader = $moduleReader;
        $this->fileDriver = $fileDriver;
        $this->logger = $logger;
    }

    /**
     * Gets convert rate from base currency
     *
     * @param string $currencyCode
     * @param \Magento\Store\Model\Store $store
     *
     * @return float
     *
     * @throws \Magento\Framework\Exception\InputException
     */
    public function getConvertRateFromBase($currencyCode, $store)
    {
        $rate = (float)$store->getBaseCurrency()->getRate($currencyCode);
        return round($rate, 2);
    }

    /**
     * Gets Magento version
     *
     * @param bool $asArray
     *
     * @return array|string
     */
    public function getVersion($asArray = false)
    {
        $versionString = $this->productMetadata->getVersion();
        return $asArray ? explode('.', $versionString) : $versionString;
    }

    /**
     * Gets extension version
     *
     * @return string
     */
    public function getExtensionVersion()
    {
        $moduleDir = $this->moduleReader->getModuleDir('', static::MODULE_CODE);

        // Read package version from module composer.json file if possible
        if ($this->fileDriver->isDirectory($moduleDir) && $this->fileDriver->isFile($moduleDir . '/composer.json')) {
            $composerFile = $this->fileDriver->fileGetContents($moduleDir . '/composer.json');

            if ($composerFile) {
                $composerFile = json_decode($composerFile, true);

                if (! empty($composerFile['version'])) {
                    return $composerFile['version'];
                }
            }
        }

        // Fallback to version from module module.xml
        $pluginVersion = $this->moduleList->getOne(self::MODULE_CODE);
        return isset($pluginVersion['setup_version']) ? $pluginVersion['setup_version'] : '';
    }

    /**
     * Curl data and return body
     *
     * @param mixed $data
     * @param string $url
     * @param string|null $token
     *
     * @return \stdClass $output
     */
    public function proxy($data, $url, $token = null)
    {
        $output = new \stdClass();
        $ch = curl_init();
        $body = json_encode($data);

        if ($token !== null) {
            $url .= "?token=$token";
        }

        curl_setopt($ch, CURLOPT_POST, 1);
        curl_setopt($ch, CURLOPT_POSTFIELDS, $body);
        curl_setopt(
            $ch,
            CURLOPT_HTTPHEADER,
            [
                'Accept: application/json',
                'Content-Type: application/json',
                'Content-Length: ' . strlen($body)
            ]
        );

        curl_setopt($ch, CURLOPT_CONNECTTIMEOUT, 60); // 2 minutes to connect
        curl_setopt($ch, CURLOPT_URL, $url);
        curl_setopt($ch, CURLOPT_HEADER, 1);
        curl_setopt($ch, CURLOPT_RETURNTRANSFER, 1);
        curl_setopt($ch, CURLOPT_SSL_VERIFYPEER, false);

        try {
            // execute
            $response = curl_exec($ch);

            $output->status = curl_getinfo($ch, CURLINFO_HTTP_CODE);
            curl_close($ch);

            // handle response
            $result = explode("\r\n\r\n", $response, 2);
            if (count($result) == 2) {
                $output->header = $result[0];
                $output->body = $result[1];
            } else {
                $output->body = 'Unexpected response';
            }
        } catch (Exception $e) {
            $this->logger->error($e);
        }

        return $output;
    }

    /**
     * Execute syncspider notification webhook based on given arguments
     *
     * @param object $collectionFactory
     * @param string $eventCode
     * @param string $entity
     * @param Observer $object
     *
     * @return void
     */
    public function runWebHook($collectionFactory, $eventCode, $entity, Observer $object)
    {
        try {
            $collection = $collectionFactory->load();

            foreach ($collection as $webHook) {
                $webHookObject = $webHook->getData();

                if ($eventCode == $webHookObject['event_code']) {
                    $eventData = $this->transform($object, $entity);
                    $eventData['data'] = 'Magento 2 Data';
                    $eventData['webhook'] = 'Magento 2 Data';
                    $eventData['info'] = [
                        'base_url' => $this->store->getBaseUrl(),
                        'server_ip' => $_SERVER['SERVER_ADDR'],
                        'time' => time()
                    ];
                    $this->proxy($eventData, $webHookObject['callback_url'], $webHookObject['token']);
                }
            }
        } catch (Exception $e) {
            $this->logger->error($e);
        }
    }

    /**
     * Transform given object based on entity identifier
     *
     * @param object $object
     * @param string $entity
     *
     * @return array
     */
    public function transform($object, $entity)
    {
        /** @var AbstractModel $data */
        $data = null;

        switch ($entity) {
            case 'product':
                $data = $object->getProduct();
                break;
            case 'category':
                $data = $object->getCategory();
                break;
            case 'customer':
                $data = $object->getCustomer();
                break;
            case 'order':
                $data = $object->getOrder();
                break;
        }

        return $data ? ['id' => $data->getId()] : [];
    }

    /**
     * Logs data to syncspider log
     *
     * @param string $message
     *
     * @return void
     */
    public function log($what)
    {
        $this->logger->info($what);
    }
}
