<?php

namespace SyncSpider\APIextension\Model\Api;

use Magento\Customer\Api\CustomerMetadataInterface;
use Magento\Customer\Api\Data\AddressInterface;
use Magento\Customer\Api\Data\CustomerInterface;
use Magento\Customer\Model\Data\Address;
use Magento\Framework\Exception\NoSuchEntityException;
use SyncSpider\APIextension\Api\CustomerRepositoryInterface;

class CustomerRepository implements CustomerRepositoryInterface
{
    /**
     * @var \Magento\Customer\Model\CustomerFactory
     */
    private $customerFactory;

    /**
     * @var \Magento\Customer\Model\CustomerRegistry
     */
    private $customerRegistry;

    /**
     * @var \Magento\Customer\Model\ResourceModel\AddressRepository
     */
    private $addressRepository;

    /**
     * @var \Magento\Customer\Api\CustomerMetadataInterface
     */
    private $customerMetadata;

    /**
     * @var \Magento\Framework\Event\ManagerInterface
     */
    private $eventManager;

    /**
     * @var \Magento\Store\Model\StoreManagerInterface
     */
    private $storeManager;

    /**
     * @var \Magento\Framework\Api\ExtensibleDataObjectConverter
     */
    private $extensibleDataObjectConverter;

    /**
     * @var \Magento\Framework\Api\ImageProcessorInterface
     */
    private $imageProcessor;

    /**
     * @param \Magento\Customer\Model\CustomerFactory $customerFactory
     * @param \Magento\Customer\Model\CustomerRegistry $customerRegistry
     * @param \Magento\Customer\Model\ResourceModel\AddressRepository $addressRepository
     * @param \Magento\Customer\Api\CustomerMetadataInterface $customerMetadata
     * @param \Magento\Framework\Event\ManagerInterface $eventManager
     * @param \Magento\Store\Model\StoreManagerInterface $storeManager
     * @param \Magento\Framework\Api\ExtensibleDataObjectConverter $extensibleDataObjectConverter
     * @param \Magento\Framework\Api\ImageProcessorInterface $imageProcessor
     */
    public function __construct(
        \Magento\Customer\Model\CustomerFactory $customerFactory,
        \Magento\Customer\Model\CustomerRegistry $customerRegistry,
        \Magento\Customer\Model\ResourceModel\AddressRepository $addressRepository,
        \Magento\Customer\Api\CustomerMetadataInterface $customerMetadata,
        \Magento\Framework\Event\ManagerInterface $eventManager,
        \Magento\Store\Model\StoreManagerInterface $storeManager,
        \Magento\Framework\Api\ExtensibleDataObjectConverter $extensibleDataObjectConverter,
        \Magento\Framework\Api\ImageProcessorInterface $imageProcessor
    ) {
        $this->customerFactory = $customerFactory;
        $this->customerRegistry = $customerRegistry;
        $this->addressRepository = $addressRepository;
        $this->customerMetadata = $customerMetadata;
        $this->eventManager = $eventManager;
        $this->storeManager = $storeManager;
        $this->extensibleDataObjectConverter = $extensibleDataObjectConverter;
        $this->imageProcessor = $imageProcessor;
    }

    /**
     * @inheritdoc
     */
    public function save(CustomerInterface $customer)
    {
        /**
         * @var \Magento\Customer\Model\Data\Customer $customer
         * @var \Magento\Customer\Model\Data\Customer $existingCustomer
         */
        try {
            $existingCustomer = $this->getByEmail($customer->getEmail(), $customer->getWebsiteId());
            $customerId = $existingCustomer->getId();
            $customer->setId($customerId);
            $prevCustomerDataArr = $existingCustomer->__toArray();
        } catch (NoSuchEntityException $e) {
            $existingCustomer = null;
            $prevCustomerDataArr = null;
            $customerId = null;
        }

        $customerArr = $customer->__toArray();
        $customer = $this->imageProcessor->save(
            $customer,
            CustomerMetadataInterface::ENTITY_TYPE_CUSTOMER,
            $existingCustomer
        );
        $origAddresses = $customer->getAddresses();
        $customer->setAddresses([]);
        $customerData = $this->extensibleDataObjectConverter->toNestedArray(
            $customer,
            [],
            CustomerInterface::class
        );

        $customer->setAddresses($origAddresses);
        /** @var \Magento\Customer\Model\Customer $customerModel */
        $customerModel = $this->customerFactory->create(['data' => $customerData]);

        $storeId = $customerModel->getStoreId();
        if ($storeId === null) {
            $customerModel->setStoreId($this->storeManager->getStore()->getId());
        }

        $customerModel->setId($customerId);

        // Need to use attribute set or future updates can cause data loss
        if (!$customerModel->getAttributeSetId()) {
            $customerModel->setAttributeSetId(CustomerMetadataInterface::ATTRIBUTE_SET_ID_CUSTOMER);
        }

        if (!array_key_exists('default_billing', $customerArr)
            && null !== $prevCustomerDataArr
            && array_key_exists('default_billing', $prevCustomerDataArr)
        ) {
            $customerModel->setDefaultBilling($prevCustomerDataArr['default_billing']);
        }

        if (!array_key_exists('default_shipping', $customerArr)
            && null !== $prevCustomerDataArr
            && array_key_exists('default_shipping', $prevCustomerDataArr)
        ) {
            $customerModel->setDefaultShipping($prevCustomerDataArr['default_shipping']);
        }

        $customerModel->save();
        $this->customerRegistry->push($customerModel);

        $customer->setId($customerModel->getId());
        $this->handleAddresses($customer, $existingCustomer === null ? [] : $existingCustomer->getAddresses());

        // remove from registry cache and reload it
        $this->customerRegistry->remove($customerModel->getId());
        $savedCustomer = $this->getById($customerModel->getId());
        $this->eventManager->dispatch(
            'customer_save_after_data_object',
            [
                'customer_data_object' => $savedCustomer,
                'orig_customer_data_object' => $customer,
                'delegate_data' => [],
            ]
        );

        return $savedCustomer;
    }

    /**
     * Retrieves customer by email
     *
     * @param string $email
     * @param int|null $websiteId
     *
     * @return CustomerInterface
     *
     * @throws \Magento\Framework\Exception\NoSuchEntityException
     */
    protected function getByEmail($email, $websiteId = null)
    {
        return $this->customerRegistry->retrieveByEmail($email, $websiteId)->getDataModel();
    }

    /**
     * Retrieves customer by id
     *
     * @param int $customerId
     *
     * @return CustomerInterface
     *
     * @throws \Magento\Framework\Exception\NoSuchEntityException
     */
    protected function getById($customerId)
    {
        return $this->customerRegistry->retrieve($customerId)->getDataModel();
    }

    /**
     * Handle given customer addresses
     *
     * @param CustomerInterface $customer
     * @param array $oldAddresses
     *
     * @throws \Magento\Framework\Exception\LocalizedException
     */
    protected function handleAddresses(CustomerInterface $customer, array $oldAddresses = [])
    {
        $customerId = $customer->getId();
        $newAddresses = $customer->getAddresses();

        if ($newAddresses === null) {
            return;
        }

        /** @var Address $address */
        foreach ($newAddresses as $address) {
            if (!$this->checkIfAddressExists($address, $oldAddresses)) {
                $address->setCustomerId($customerId)->setRegion($address->getRegion());
            }

            $this->addressRepository->save($address);
        }
    }

    /**
     * Checks if address already exists
     *
     * @param Address $address
     * @param Address[] $oldAddresses
     *
     * @return bool
     */
    private function checkIfAddressExists(Address $address, array $oldAddresses)
    {
        foreach ($oldAddresses as $oldAddress) {
            if ($oldAddress->getCity() === $address->getCity()
                && $oldAddress->getCountryId() == $address->getCountryId()
                && $oldAddress->getPostcode() === $address->getPostcode()
                && $oldAddress->getRegionId() == $address->getRegionId()
                && implode("\n", $oldAddress->getStreet()) === implode("\n", $address->getStreet())
            ) {
                $oldData = $oldAddress->__toArray();
                $newData = $address->__toArray();
                foreach ($oldData as $key => $value) {
                    if (!isset($newData[$key]) && $key !== AddressInterface::REGION) {
                        $address->setData($key, $value);
                    } elseif ($key === AddressInterface::REGION) {
                        $address->setRegion($oldAddress->getRegion());
                    }
                }

                return true;
            }
        }

        return false;
    }
}
