<?php

namespace SyncSpider\APIextension\Model\Api;

use Magento\Framework\App\ResourceConnection;
use Magento\Framework\EntityManager\MetadataPool;
use SyncSpider\APIextension\Api\EntityTypeRepositoryInterface;
use DomainException;
use Exception;

class EntityTypeRepository implements EntityTypeRepositoryInterface
{
    /**
     * @var ResourceConnection
     */
    private $resourceConnection;

    /**
     * @var MetadataPool
     */
    private $metadataPool;

    /**
     * @var string[]
     */
    private $entityTypeMap;

    /**
     * @var [][]
     */
    private $cache = [];

    /**
     * Entity type repository constructor.
     *
     * @param ResourceConnection $resourceConnection
     * @param MetadataPool $metadataPool
     * @param string[] $entityTypeMap
     */
    public function __construct(
        ResourceConnection $resourceConnection,
        MetadataPool $metadataPool,
        array $entityTypeMap = []
    ) {
        $this->resourceConnection = $resourceConnection;
        $this->metadataPool = $metadataPool;
        $this->entityTypeMap = $entityTypeMap;
    }

    /**
     * @inheritDoc
     */
    public function getEntityTypeData($entityType): array
    {
        $entityTypeCode = $entityType;

        if (interface_exists($entityType)) {
            try {
                $metadata = $this->metadataPool->getMetadata($entityType);
                $entityTypeCode = $metadata->getEavEntityType();
            } catch (Exception $e) {
                // suppress exceptions
            }
        }

        if (! array_key_exists($entityTypeCode, $this->cache)) {
            $this->cache[$entityTypeCode] = null;

            if ($entityTypeCode) {
                $connection = $this->resourceConnection->getConnection();
                $select = $connection->select()
                    ->from(
                        ['t' => $this->resourceConnection->getTableName('eav_entity_type')],
                        ['t.entity_type_id', 't.entity_type_code']
                    )
                    ->limit(1);

                if (is_numeric($entityType)) {
                    $select->where('t.entity_type_id = :entity_type_code');
                } else {
                    $select->where('t.entity_type_code = :entity_type_code');
                }

                $row = $connection->fetchRow($select, ['entity_type_code' => $entityTypeCode]);

                if ($row && ! empty($row['entity_type_id'])) {
                    $this->cache[$row['entity_type_id']] = $row;
                    $this->cache[$row['entity_type_code']] = $row;
                }
            }
        }

        if ($this->cache[$entityTypeCode] === null) {
            throw new DomainException(
                'Given entity type "' . $entityType
                . '" does not belong to any valid eav entity type code or entity data interface'
            );
        }

        return $this->cache[$entityTypeCode];
    }

    /**
     * @inheritDoc
     */
    public function getEntityTypeId($entityType): int
    {
        $entityTypeData = $this->getEntityTypeData($entityType);
        return $entityTypeData['entity_type_id'];
    }

    /**
     * @inheritDoc
     */
    public function getEntityTypeCode($entityType): string
    {
        $entityTypeData = $this->getEntityTypeData($entityType);
        return $entityTypeData['entity_type_code'];
    }

    /**
     * @inheritDoc
     */
    public function getEntityType($entityType): ?string
    {
        try {
            $entityTypeCode = $this->getEntityTypeCode($entityType);
            $entityType = $this->entityTypeMap[$entityTypeCode] ?? null;

            if (! $entityType
                || ! interface_exists($entityType)
                && ! class_exists($entityType)
            ) {
                $entityType = null;
            }

            return $entityType;
        } catch (Exception $e) {
            // suppress exception
        }

        return null;
    }
}
