<?php

namespace SyncSpider\APIextension\Model\Api;

use Magento\Framework\Api\SearchCriteriaInterface;
use Magento\Framework\Api\SortOrder;
use Magento\Sales\Model\ResourceModel\Order\Status\Collection as OrderStatusCollection;
use Magento\Sales\Model\ResourceModel\Order\Status\CollectionFactory as OrderStatusCollectionFactory;
use SyncSpider\APIextension\Api\OrderStatusRepositoryInterface;
use SyncSpider\APIextension\Api\Data\OrderStatusSearchResultInterface;
use SyncSpider\APIextension\Api\Data\OrderStatusSearchResultInterfaceFactory;

class OrderStatusRepository implements OrderStatusRepositoryInterface
{
    /**
     * @var OrderStatusCollectionFactory $statusCollectionFactory
     */
    private $statusCollectionFactory;

    /**
     * @var OrderStatusSearchResultInterfaceFactory
     */
    private $orderStatusSearchResultInterfaceFactory;

    /**
     * @param OrderStatusCollectionFactory $statusCollectionFactory
     * @param OrderStatusSearchResultInterfaceFactory $orderStatusSearchResultInterfaceFactory
     */
    public function __construct(
        OrderStatusCollectionFactory $statusCollectionFactory,
        OrderStatusSearchResultInterfaceFactory $orderStatusSearchResultInterfaceFactory
    ) {
        $this->statusCollectionFactory = $statusCollectionFactory;
        $this->orderStatusSearchResultInterfaceFactory = $orderStatusSearchResultInterfaceFactory;
    }

    /**
     * @inheritDoc
     */
    public function getList(SearchCriteriaInterface $searchCriteria)
    {
        $collection = $this->statusCollectionFactory->create();
        $collection->joinStates();
        $this->addFiltersToCollection($searchCriteria, $collection);
        $this->addSortOrdersToCollection($searchCriteria, $collection);
        $this->addPagingToCollection($searchCriteria, $collection);
        $collection->load();
        return $this->buildSearchResult($searchCriteria, $collection);
    }

    /**
     * Add filters from given search criteria to given collecction
     *
     * @param SearchCriteriaInterface $searchCriteria
     * @param OrderStatusCollection $collection
     *
     * @return void
     */
    private function addFiltersToCollection(SearchCriteriaInterface $searchCriteria, OrderStatusCollection $collection)
    {
        foreach ($searchCriteria->getFilterGroups() as $filterGroup) {
            $fields = $conditions = [];
            foreach ($filterGroup->getFilters() as $filter) {
                $fields[] = $filter->getField();
                $conditions[] = [$filter->getConditionType() => $filter->getValue()];
            }
            $collection->addFieldToFilter($fields, $conditions);
        }
    }

    /**
     * Add sort orders from given search criteria to given collecction
     *
     * @param SearchCriteriaInterface $searchCriteria
     * @param OrderStatusCollection $collection
     *
     * @return void
     */
    private function addSortOrdersToCollection(SearchCriteriaInterface $searchCriteria, OrderStatusCollection $collection)
    {
        foreach ((array)$searchCriteria->getSortOrders() as $sortOrder) {
            $direction = $sortOrder->getDirection() == SortOrder::SORT_ASC ? 'asc' : 'desc';
            $collection->addOrder($sortOrder->getField(), $direction);
        }
    }

    /**
     * Add pagination from given search criteria to given collecction
     *
     * @param SearchCriteriaInterface $searchCriteria
     * @param OrderStatusCollection $collection
     *
     * @return void
     */
    private function addPagingToCollection(SearchCriteriaInterface $searchCriteria, OrderStatusCollection $collection)
    {
        $collection->setPageSize($searchCriteria->getPageSize());
        $collection->setCurPage($searchCriteria->getCurrentPage());
    }

    /**
     * Build order status search result
     *
     * @param SearchCriteriaInterface $searchCriteria
     * @param OrderStatusCollection $collection
     *
     * @return OrderStatusSearchResultInterface
     */
    private function buildSearchResult(SearchCriteriaInterface $searchCriteria, OrderStatusCollection $collection)
    {
        $searchResults = $this->orderStatusSearchResultInterfaceFactory->create();
        $searchResults->setSearchCriteria($searchCriteria);
        $searchResults->setItems($collection->getData());
        $searchResults->setTotalCount($collection->getSize());
        return $searchResults;
    }
}
