<?php

namespace SyncSpider\APIextension\Model\Api;

use Magento\Catalog\Model\Category\Attribute;
use Magento\Framework\Exception\InputException;
use Magento\Framework\Exception\StateException;
use Magento\Ui\Component\Form\Element\DataType\Price;
use Magento\Ui\Component\Form\Element\MultiSelect;
use Magento\Ui\Component\Form\Element\Select;
use SyncSpider\APIextension\Api\ProductAttributeRepositoryInterface;

class ProductAttributeRepository implements ProductAttributeRepositoryInterface
{
    public const EXCLUDED_ATTRIBUTES = [
        'price_view',
        'shipment_type',
        'links_title',
        'price_type',
        'samples_title',
        'links_purchased_separately',
        'quantity_and_stock_status',
        'sku_type',
        'weight_type',
        'updated_at',
        'created_at',
        'tier_price',
        'swatch_image',
        'category_ids',
    ];

    /**
     * @var \Magento\Catalog\Model\Product\AttributeSet\Options
     */
    private $attributeSetOptions;

    /**
     * @var \Magento\Store\Model\StoreRepository
     */
    private $storeRepository;

    /**
     * @var \Magento\Eav\Api\AttributeOptionManagementInterface
     */
    private $eavOptionManagement;

    /**
     * @var \Magento\Catalog\Model\ResourceModel\Product\Collection
     */
    private $attributeCollection;

    /**
     * @var \Magento\Catalog\Model\ProductFactory
     */
    private $productFactory;

    /**
     * @var \Magento\ConfigurableProduct\Model\ConfigurableAttributeHandler
     */
    private $configurableAttributeHandler;

    /**
     * ProductAttributeRepository constructor.
     *
     * @param \Magento\Catalog\Model\ResourceModel\Product\Attribute\Collection $collection
     * @param \Magento\Eav\Api\AttributeOptionManagementInterface $eavOptionManagement
     * @param \Magento\Catalog\Model\Product\AttributeSet\Options $attributeSetOptions
     * @param \Magento\Store\Model\StoreRepository $storeRepository
     * @param \Magento\Catalog\Model\ProductFactory $productFactory
     * @param \Magento\ConfigurableProduct\Model\ConfigurableAttributeHandler $configurableAttributeHandler
     */
    public function __construct(
        \Magento\Catalog\Model\ResourceModel\Product\Attribute\Collection $collection,
        \Magento\Eav\Api\AttributeOptionManagementInterface $eavOptionManagement,
        \Magento\Catalog\Model\Product\AttributeSet\Options $attributeSetOptions,
        \Magento\Store\Model\StoreRepository $storeRepository,
        \Magento\Catalog\Model\ProductFactory $productFactory,
        \Magento\ConfigurableProduct\Model\ConfigurableAttributeHandler $configurableAttributeHandler
    ) {
        $this->attributeSetOptions = $attributeSetOptions;
        $this->storeRepository = $storeRepository;
        $this->eavOptionManagement = $eavOptionManagement;
        $this->attributeCollection = $collection;
        $this->productFactory = $productFactory;
        $this->configurableAttributeHandler = $configurableAttributeHandler;
    }

    /**
     * @inheritDoc
     */
    public function getAttributes()
    {
        $result = [];

        $attributes = $this->getProductAttributes()->load();
        $requiredAttributes = $this->getRequiredFieldsForDefaultSet();

        /** @var Attribute $attribute */
        foreach ($attributes as $attribute) {
            if (!$this->isAttributeApplicable($attribute)) {
                continue;
            }

            $isCreateDynamicallyAllowed = false;
            $attributeCode = $attribute->getName();
            $attributeType = $attribute->getFrontendInput();
            $options = $this->getAttributeOptions($attributeCode);

            if ($attributeType === Select::NAME || $attributeType === MultiSelect::NAME) {
                $isCreateDynamicallyAllowed = !$this->canManageOptionDefaultOnly($attribute);
            }

            $result[$attributeCode] = [
                'attribute_code' => $attributeCode,
                'attribute_label' => $attribute->getDefaultFrontendLabel(),
                'attribute_type' => $attributeType,
                'is_create_dynamic_allowed' => $isCreateDynamicallyAllowed,
                'required' => in_array($attributeCode, $requiredAttributes),
                'options' => $options,
                'scope' => $attribute->getScope()
            ];
        }

        return array_merge($result, $this->getAdditionalAttributes());
    }

    /**
     * @inheritDoc
     */
    public function getConfigurableAttributes($attributeSetId)
    {
        $result = [];

        $configurableAttributes = $this->getConfigurableAttributeCodes($attributeSetId);
        foreach ($this->getProductAttributes() as $attribute) {
            $code = $attribute->getName();
            $result[$code] = [
                'id' => $attribute->getId(),
                'attribute_code' => $code,
                'attribute_type' => $attribute->getFrontendInput(),
                'is_global' => $attribute->getData('is_global'),
                'is_configurable' => in_array($code, $configurableAttributes),
                'frontend_input' => $attribute->getFrontendInput(),
                'apply_to' => $attribute->getApplyTo(),
                'required' => (bool)$attribute->getIsRequired(),
                'options' => $this->getAttributeOptions($code),
            ];
        }

        return $result;
    }

    /**
     * Get list of store views
     *
     * @return array
     */
    public function getStoreOptions()
    {
        $storeList = [];

        /** @var \Magento\Store\Model\Store $store */
        foreach ($this->storeRepository->getList() as $store) {
            if (!$store->isActive() || ($store->getCode() === 'admin')) {
                continue;
            }

            $storeList[$store->getCode()] = $store->getName();
        }

        $storeList['all'] = __('All Store Views');

        return $storeList;
    }

    /**
     * Get attribute options for given attribute code
     *
     * @param string $attributeCode
     *
     * @return string[]
     */
    public function getAttributeOptions($attributeCode)
    {
        $result = [];

        try {
            $options = $this->eavOptionManagement->getItems(
                \Magento\Catalog\Api\Data\ProductAttributeInterface::ENTITY_TYPE_CODE,
                $attributeCode
            );

            foreach ($options as $optionInstance) {
                if ($optionInstance->getLabel() && $optionInstance->getValue() !== '') {
                    $result[$optionInstance->getValue()] = $optionInstance->getLabel();
                }
            }
        } catch (InputException | StateException $e) {
            // suppress exceptions
        }

        return $result;
    }

    /**
     * Get product attributes collection
     *
     * @param bool $onlyRequired
     * @param int|null $attributeSetId
     *
     * @return \Magento\Catalog\Model\ResourceModel\Product\Attribute\Collection
     */
    private function getProductAttributes($onlyRequired = false, $attributeSetId = null)
    {
        $attributeCollection = $this->attributeCollection->addFieldToFilter(
            'main_table.attribute_code',
            ['nin' => self::EXCLUDED_ATTRIBUTES]
        )->addFieldToFilter(
            'main_table.frontend_input',
            ['nin' => ['gallery', 'media_image']]
        )->addVisibleFilter();

        if ($onlyRequired) {
            $attributeCollection->addFieldToFilter('main_table.is_required', true);
        }

        if ($attributeSetId) {
            $attributeCollection->setAttributeSetFilter($attributeSetId);
        }

        return clone $attributeCollection;
    }

    /**
     * Get list of required product attributes in default attribute set
     *
     * @return array
     */
    private function getRequiredFieldsForDefaultSet()
    {
        $defaultAttributeSetId = $this->productFactory->create()->getDefaultAttributeSetId();
        $attributes = $this->getProductAttributes(true, $defaultAttributeSetId)->load()->toArray();
        return array_column($attributes['items'], 'attribute_code');
    }

    /**
     * Filters attributes by criteria
     *
     * @param Attribute $attribute
     *
     * @return bool
     */
    private function isAttributeApplicable($attribute)
    {
        $applyTo = $attribute->getApplyTo();

        return empty($applyTo) ||
            in_array(\Magento\ConfigurableProduct\Model\Product\Type\Configurable::TYPE_CODE, $applyTo) ||
            in_array(\Magento\Catalog\Model\Product\Type::TYPE_SIMPLE, $applyTo);
    }

    /**
     * Get list of available product attribute sets
     *
     * @return array
     */
    private function getAttributeSetOptions()
    {
        return array_column($this->attributeSetOptions->toOptionArray(), 'label', 'value');
    }

    /**
     * Returns attribute set and store options
     *
     * @return array
     */
    private function getAdditionalAttributes()
    {
        $result = [];
        $this->addField($result, 'attribute_set_id', 'Attribute Set Id', Select::NAME, true, $this->getAttributeSetOptions());
        $this->addField($result, 'store_ids', 'Store Ids', MultiSelect::NAME, true, $this->getStoreOptions());
        $this->addField($result, 'price', 'Price Incl. Tax', Price::NAME);
        $this->addField($result, 'price_excl_tax', 'Price Excl. Tax', Price::NAME);
        $this->addField($result, 'final_price_incl_tax', 'Final Price Incl. Tax', Price::NAME);
        $this->addField($result, 'final_price_excl_tax', 'Final Price Excl. Tax', Price::NAME);
        return $result;
    }

    /**
     * Add attribute data to given attribute data array
     *
     * @param array $data
     * @param string $code
     * @param string $label
     * @param string $type
     * @param bool $required
     * @param array $options
     * @param bool $dynamically
     *
     * @return void
     */
    private function addField(&$data, $code, $label, $type, $required = false, $options = [], $dynamically = false)
    {
        $data[$code] = [
            'attribute_code' => $code,
            'attribute_label' => __($label),
            'attribute_type' => $type,
            'is_create_dynamic_allowed' => $dynamically,
            'options' => $options,
            'required' => $required,
        ];
    }

    /**
     * Get list of configurable attribute codes
     *
     * @param int $attributeSetId
     *
     * @return array
     */
    private function getConfigurableAttributeCodes($attributeSetId)
    {
        $result = [];

        $configurableAttributeCollection = $this->configurableAttributeHandler->getApplicableAttributes()
            ->setAttributeSetFilter($attributeSetId);

        /** @var \Magento\Catalog\Api\Data\ProductAttributeInterface $attribute */
        foreach ($configurableAttributeCollection->getItems() as $attribute) {
            if ($this->configurableAttributeHandler->isAttributeApplicable($attribute)) {
                $result[] = $attribute->getAttributeCode();
            }
        }

        return $result;
    }

    /**
     * Is true only for system attributes which use source model
     *
     * Option labels and position for such attributes are kept in source model and thus cannot be overridden
     *
     * @param Attribute $attribute
     *
     * @return bool
     */
    public function canManageOptionDefaultOnly($attribute)
    {
        return !$attribute->getCanManageOptionLabels() &&
            !$attribute->getIsUserDefined() &&
            $attribute->getSourceModel();
    }
}
