<?php

namespace SyncSpider\APIextension\Model\Api;

use DomainException;
use Magento\Framework\App\ResourceConnection;
use Magento\Framework\EntityManager\{
    Operation\ExtensionPool,
    EntityManager,
    HydratorPool
};
use SyncSpider\APIextension\Api\{
    Data\SynchronizationDataInterface,
    EntityTypeRepositoryInterface,
    SynchronizationDataRepositoryInterface
};

class SynchronizationDataRepository implements SynchronizationDataRepositoryInterface
{
    /**
     * @var EntityManager
     */
    private $entityManager;

    /**
     * @var HydratorPool
     */
    private $hydratorPool;

    /**
     * @var ExtensionPool
     */
    private $extensionPool;

    /**
     * @var EntityTypeRepositoryInterface
     */
    private $entityTypeRepository;

    /**
     * @var ResourceConnection
     */
    private $resourceConnection;

    /**
     * Order synchronization data repository constructor.
     *
     * @param EntityManager $entityManager
     * @param HydratorPool $hydratorPool
     * @param ExtensionPool $extensionPool
     * @param EntityTypeRepositoryInterface $entityTypeRepository
     * @param ResourceConnection $resourceConnection
     */
    public function __construct(
        EntityManager $entityManager,
        HydratorPool $hydratorPool,
        ExtensionPool $extensionPool,
        EntityTypeRepositoryInterface $entityTypeRepository,
        ResourceConnection $resourceConnection
    ) {
        $this->entityManager = $entityManager;
        $this->hydratorPool = $hydratorPool;
        $this->extensionPool = $extensionPool;
        $this->entityTypeRepository = $entityTypeRepository;
        $this->resourceConnection = $resourceConnection;
    }

    /**
     * @inheritDoc
     */
    public function save(
        SynchronizationDataInterface $synchronizationData,
        string $entityType,
        array $entityIds = []
    ): int {
        $synchronizationData = $this->create($synchronizationData, $entityType);

        if (! $synchronizationData->getExecutionId()) {
            throw new DomainException('Unable to create synchorinzation data entry in database for given entity type');
        }

        return $this->attachEntityIds($synchronizationData, $entityIds);
    }

    /**
     * Create synchorinzation data entry in database for given entity type
     *
     * @param SynchronizationDataInterface $synchronizationData
     * @param string $entityType
     *
     * @return SynchronizationDataInterface
     */
    private function create(
        SynchronizationDataInterface $synchronizationData,
        string $entityType
    ): SynchronizationDataInterface {
        $synchronizationData->setEntityTypeId(
            $this->entityTypeRepository->getEntityTypeId($entityType)
        );
        $connection = $this->resourceConnection->getConnection();
        $connection->insertOnDuplicate(
            $this->resourceConnection->getTableName('syncspider_synchronization_data'),
            [
                $this->hydratorPool->getHydrator(SynchronizationDataInterface::class)->extract($synchronizationData)
            ],
            []
        );
        return $this->entityManager->load($synchronizationData, $synchronizationData->getExecutionId());
    }

    /**
     * Attach given entity ids to given synchorinzation data
     *
     * @param SynchronizationDataInterface $synchronizationData
     * @param string[] $entityIds
     *
     * @return integer
     */
    private function attachEntityIds(SynchronizationDataInterface $synchronizationData, array $entityIds = []): int
    {
        $rows = [];
        $affectedRows = 0;
        $connection = $this->resourceConnection->getConnection();

        foreach ($entityIds as $entityId) {
            $rows[] = [
                SynchronizationDataInterface::EXECUTION_ID => $synchronizationData->getExecutionId(),
                'entity_id' => $entityId
            ];
        }

        if (! empty($rows)) {
            // insert affected entities
            $affectedRows = $connection->insertOnDuplicate(
                $this->resourceConnection->getTableName('syncspider_synchronization_data_entities'),
                $rows,
                []
            );

            // process additional synchronization data attaching to entities
            $entityType = $this->entityTypeRepository->getEntityType($synchronizationData->getEntityTypeId());

            if ($entityType) {
                $actions = $this->extensionPool->getActions($entityType, 'syncspider_synchronization_data');

                foreach ($actions as $action) {
                    $action->execute($synchronizationData, [
                        'entityIds' => $entityIds
                    ]);
                }
            }
        }

        return $affectedRows;
    }
}
