<?php

namespace SyncSpider\APIextension\Model;

use Magento\Framework\Api\{Filter, SearchCriteriaInterface};
use Magento\Framework\Api\Search\FilterGroup;
use Magento\Framework\Data\Collection\AbstractDb;
use SyncSpider\APIextension\Api\{
    FilterMetadataInterface,
    FilterRendererInterface,
    FilterGroupProcessorInterface,
    FilterMetadataInterfaceFactory
};
use Closure;
use Exception;

class DirectFilterGroupProcessor implements FilterGroupProcessorInterface
{
    /**
     * @var FilterRendererInterface
     */
    private $filterRenderer;

    /**
     * @var FilterMetadataInterfaceFactory
     */
    private $filterMetadataFactory;

    /**
     * @var FilterMetadataInterface[]
     */
    private $filtersMetadata;

    /**
     * @var array
     */
    private $filtersData;

    /**
     * @var Closure
     */
    private $mergeBinds;

    /**
     * @var Closure
     */
    private $getBinds;

    /**
     * OrderBillingAddressProcessor constructor.
     *
     * @param FilterRendererInterface $filterRenderer
     * @param FilterMetadataInterfaceFactory $filterMetadataFactory
     * @param array array $filterMap
     */
    public function __construct(
        FilterRendererInterface $filterRenderer,
        FilterMetadataInterfaceFactory $filterMetadataFactory,
        array $filtersData = []
    ) {
        $this->filterRenderer = $filterRenderer;
        $this->filterMetadataFactory = $filterMetadataFactory;
        $this->filtersData = $filtersData;
        $this->mergeBinds = function (array $binds) {
            if (property_exists($this, '_bindParams') && is_array($this->_bindParams)) {
                $this->_bindParams = array_merge($this->_bindParams, $binds);
            }
        };
        $this->getBinds = function () {
            if (property_exists($this, '_bindParams') && is_array($this->_bindParams)) {
                return $this->_bindParams;
            }

            return [];
        };
    }

    /**
     * Get filters metadata
     *
     * Parse "filtersData" property into list of filter metadata interfaces
     *
     * @return FilterMetadataInterface[]
     */
    public function filtersMetadata(): array
    {
        if (! isset($this->filtersMetadata)) {
            $this->filtersMetadata = [];

            foreach ($this->filtersData as $key => $data) {
                $this->filtersMetadata[] = $this->filterMetadataFactory->create([
                    'tableName' => $data['tableName'] ?? 'main_table',
                    'tableAlias' => $data['tableAlias'] ?? 'main_table',
                    'fieldPrefix' => $data['fieldPrefix'],
                    'constraint' => '',
                    'column' => $data['column']
                ]);
            }
        }

        return $this->filtersMetadata;
    }

    /**
     * Whether given metadata handles given filter
     *
     * @param FilterMetadataInterface $metadata
     * @param Filter $filter
     *
     * @return boolean
     */
    public function metadataHandlesFilter(FilterMetadataInterface $metadata, Filter $filter): bool
    {
        if (strpos($filter->getField(), $metadata->getFieldPrefix()) !== 0) {
            return false;
        }

        return true;
    }

    /**
     * @inheritDoc
     */
    public function handles(FilterGroup $filterGroup): bool
    {
        // Check that given filter group filters is handled by at least one filter metadata
        foreach ($filterGroup->getFilters() as $filter) {
            foreach ($this->filtersMetadata() as $metadata) {
                if ($this->metadataHandlesFilter($metadata, $filter)) {
                    continue 2;
                }
            }

            return false;
        }

        return true;
    }

    /**
     * @inheritDoc
     */
    public function apply(AbstractDb $collection, SearchCriteriaInterface $searchCriteria): AbstractDb
    {
        if (count($searchCriteria->getFilterGroups()) > 0) {
            $select = $collection->getConnection()->select();
            // apply suffix bind name so it can be used inside filter renderer to assign unique bind names
            $select->bind([
                FilterRendererInterface::SUFFIX_BIND_NAME => count($this->getBinds->call($collection))
            ]);

            foreach ($searchCriteria->getFilterGroups() as $group) {
                if (! $this->handles($group)) {
                    $groupJson = json_encode($group->__toArray());
                    throw new Exception(
                        'Filter group processor ' . static::class . ' does not handle given filter group: ' . $groupJson
                    );
                }

                $conditions = [];

                foreach ($group->getFilters() as $filter) {
                    foreach ($this->filtersMetadata() as $metadata) {
                        if ($this->metadataHandlesFilter($metadata, $filter)) {
                            $conditions[] = $this->filterRenderer->render($filter, $select, $metadata);
                            break;
                        }
                    }
                }

                // attach conditions to main select as direct constraint
                $collection->getSelect()->where(sprintf('%s', implode(' OR ', $conditions)));
            }

            // apply binds to collection by removing the suffix bind name
            $binds = $select->getBind();
            unset($binds[FilterRendererInterface::SUFFIX_BIND_NAME]);
            $this->mergeBinds->call($collection, $binds);
        }

        return $collection;
    }
}
