<?php

namespace SyncSpider\APIextension\Model\ExtensionAttributes\Processor;

use Magento\Framework\Api\{
    ExtensibleDataInterface,
    AbstractSimpleObject,
    SimpleDataObjectConverter
};
use Magento\Sales\Api\{
    Data\OrderInterface,
    Data\OrderAddressInterface,
    Data\OrderAddressExtensionInterface,
    Data\ShippingAssignmentInterface
};
use SyncSpider\APIextension\Api\{
    ExtensionAttributes\ProcessorContextInterface,
    ExtensionAttributes\ProcessorInterface
};
use SyncSpider\APIextension\Model\ExtensionAttributes\SalesOrderContext;

class OrderAddressExtensionAttributesProcessor implements ProcessorInterface
{
    /**
     * @var SalesOrderContext
     */
    private $orderContext;

    public function __construct(SalesOrderContext $orderContext)
    {
        $this->orderContext = $orderContext;
    }

    /**
     * Get shipping address for given order
     *
     * @param OrderInterface $order
     *
     * @return OrderAddressInterface|null
     */
    public function getShippingAddress(OrderInterface $order): ?OrderAddressInterface
    {
        if (method_exists($order, 'getShippingAddress')) {
            $shippingAdress = $order->getShippingAddress();

            if ($shippingAdress instanceof OrderAddressInterface) {
                return $shippingAdress;
            }
        }

        if ($order
            && $order->getExtensionAttributes()
            && is_array($order->getExtensionAttributes()->getShippingAssignments())
            && $order->getExtensionAttributes()->getShippingAssignments()[0] instanceof ShippingAssignmentInterface
        ) {
            $shippingAdress = $order->getExtensionAttributes()->getShippingAssignments()[0]->getShipping();

            if ($shippingAdress instanceof OrderAddressInterface) {
                return $shippingAdress;
            }
        }

        return null;
    }

    /**
     * Process given order address and order address extension attributes
     *
     * @param OrderAddressInterface $address
     * @param OrderAddressExtensionInterface|null $extensionAttributes
     *
     * @return OrderAddressInterface
     */
    public function processAddressExtensionAttributes(
        OrderAddressInterface $address,
        OrderAddressExtensionInterface $extensionAttributes = null
    ): OrderAddressInterface {
        $addressExtension = $address->getExtensionAttributes();

        if ($extensionAttributes instanceof AbstractSimpleObject) {
            $extensionData = $extensionAttributes->__toArray();

            foreach ($extensionData as $extensionAttribute => $value) {
                // sync extension attributes with address extension attributes
                if ($addressExtension instanceof OrderAddressExtensionInterface) {
                    $setter = SimpleDataObjectConverter::snakeCaseToCamelCase('set_' . $extensionAttribute);
                    $getter = SimpleDataObjectConverter::snakeCaseToCamelCase('get_' . $extensionAttribute);

                    if (method_exists($addressExtension, $setter)
                        && method_exists($extensionAttributes, $getter)
                    ) {
                        $addressExtension->{$setter}($extensionAttributes->{$getter}());
                    }
                }
            }
        }

        return $address;
    }

    /**
     * Populate order "ext_order_id" field
     *
     * Value is set to syncspider order peyment external order id
     *
     * @param ExtensibleDataInterface $entity
     * @param ProcessorContextInterface $context
     *
     * @return ExtensibleDataInterface
     */
    public function process(ExtensibleDataInterface $entity, ProcessorContextInterface $context): ExtensibleDataInterface
    {
        if ($entity instanceof OrderInterface) {
            $syncspiderOrder = $this->orderContext->getSyncspiderOrder($context);

            if ($syncspiderOrder) {
                $billingAddress = $entity->getBillingAddress();
                $shippingAdress = $this->getShippingAddress($entity);

                if ($billingAddress && $syncspiderOrder->getBillingAddress()) {
                    $this->processAddressExtensionAttributes(
                        $billingAddress,
                        $syncspiderOrder->getBillingAddress()->getExtensionAttributes()
                    );
                }

                if ($shippingAdress && $syncspiderOrder->getShippingAddress()) {
                    $this->processAddressExtensionAttributes(
                        $shippingAdress,
                        $syncspiderOrder->getShippingAddress()->getExtensionAttributes()
                    );
                }
            }
        }

        return $entity;
    }
}