<?php

namespace SyncSpider\APIextension\Model\ExtensionAttributes\Processor;

use Magento\Framework\{
    Api\ExtensibleDataInterface,
    Exception\LocalizedException
};
use Magento\Sales\Api\Data\{
    OrderInterface,
    OrderStatusHistoryInterface
};
use Magento\Sales\Model\Order\{
    Status\History,
    Status
};
use Magento\Sales\Model\ResourceModel\Order\Status\CollectionFactory;
use SyncSpider\APIextension\Api\{
    ExtensionAttributes\ProcessorContextInterface,
    ExtensionAttributes\ProcessorInterface
};
use SyncSpider\APIextension\Model\ExtensionAttributes\SalesOrderContext;

class OrderStatusProcessor implements ProcessorInterface
{
    /**
     * @var SalesOrderContext
     */
    private $orderContext;

    /**
     * @var CollectionFactory
     */
    private $statusCollectionFactory;

    public function __construct(
        SalesOrderContext $orderContext,
        CollectionFactory $statusCollectionFactory
    ) {
        $this->orderContext = $orderContext;
        $this->statusCollectionFactory = $statusCollectionFactory;
    }

    /**
     * Change order status and state based on the syncspider order data extension attribute
     *
     * @param ExtensibleDataInterface $entity
     * @param ProcessorContextInterface $context
     *
     * @return ExtensibleDataInterface
     */
    public function process(ExtensibleDataInterface $entity, ProcessorContextInterface $context): ExtensibleDataInterface
    {
        if ($entity instanceof OrderInterface) {
            $syncspiderOrder = $this->orderContext->getSyncspiderOrder($context);

            if ($syncspiderOrder) {
                $newStatus = $syncspiderOrder->getStatus();

                if ($newStatus && $entity->getStatus() !== $newStatus) {
                    $status = $this->getOrderStatus($newStatus);

                    if (! $status || ! $status->getStatus()) {
                        throw new LocalizedException(
                            __("Status %1 not found.", [$newStatus])
                        );
                    }

                    $previousStatus = $entity->getStatus();
                    $entity->setStatus($status->getStatus());
                    $entity->setState($status->getState());

                    if (method_exists($entity, 'addCommentToStatusHistory')) {
                        $statusHistory = $entity->addCommentToStatusHistory(
                            "SyncSpider order status automatically set from [{$previousStatus}] to [{$status->getStatus()}]",
                            false,
                            false
                        );

                        if ($statusHistory instanceof OrderStatusHistoryInterface) {
                            $statusHistory->setIsCustomerNotified(History::CUSTOMER_NOTIFICATION_NOT_APPLICABLE);
                        }
                    }
                }
            }
        }

        return $entity;
    }

    /**
     * Get order staus for given status
     *
     * @param string $status
     *
     * @return Status|null
     */
    public function getOrderStatus(string $status): ?Status
    {
        $status = $this->statusCollectionFactory->create()
            ->joinStates()
            ->addFieldToFilter('main_table.status', $status)
            ->setPageSize(1)
            ->getLastItem();
        return $status;
    }
}