<?php

namespace SyncSpider\APIextension\Model\ExtensionAttributes;

use Magento\Sales\Api\Data\{OrderInterface, OrderExtensionInterface};
use Magento\Quote\Api\Data\CartInterface;
use SyncSpider\APIextension\Api\{
    Data\OrderInterface as SyncSpiderOrderInterface,
    ExtensionAttributes\ProcessorContextInterface,
    ExtensionAttributes\ProcessorContextInterfaceFactory
};

class SalesOrderContext
{
    public const SYNCSPIDER_ORDER_CONTEXT = 'syncspider_order';

    public const QUOTE_CONTEXT = 'sales_order_quote';

    public const ORDER_CONTEXT = 'sales_order';

    /**
     * @var ProcessorContextInterfaceFactory
     */
    private $contextFactory;

    public function __construct(ProcessorContextInterfaceFactory $contextFactory)
    {
        $this->contextFactory = $contextFactory;
    }

    /**
     * Get processor context for given order extension
     *
     * @param ProcessorContextInterface $context
     *
     * @return ProcessorContextInterface|null
     */
    public function getProcessorContext(ProcessorContextInterface $context): ?ProcessorContextInterface
    {
        $orderExtension = $this->getOrderExtension($context);

        if ($orderExtension && method_exists($orderExtension, 'getSyncspiderProcessorContext')) {
            $context = $orderExtension->getSyncspiderProcessorContext();

            if ($context instanceof ProcessorContextInterface) {
                return $context;
            }
        }

        return null;
    }

    /**
     * Inherit context for given context
     *
     * @param ProcessorContextInterface $context
     *
     * @return ProcessorContextInterface
     */
    public function inheritContext(ProcessorContextInterface $context): ProcessorContextInterface
    {
        $inheritContext = $this->getProcessorContext($context);

        if ($inheritContext && $inheritContext !== $context) {
            $context->setContext(array_merge(
                $inheritContext->retrieveContext(),
                $context->retrieveContext()
            ));
        }

        return $context;
    }

    /**
     * Create processor context for given syncspider order
     *
     * @param SyncSpiderOrderInterface $syncspiderOrder
     *
     * @return ProcessorContextInterface
     */
    public function createProcessorContext(SyncSpiderOrderInterface $syncspiderOrder): ProcessorContextInterface
    {
        $context = $this->contextFactory->create([
            'context' => [
                static::SYNCSPIDER_ORDER_CONTEXT => $syncspiderOrder
            ]
        ]);
        return $this->inheritContext($context);
    }

    /**
     * Create processor context for given quote
     *
     * @param CartInterface $quote
     *
     * @return ProcessorContextInterface
     */
    public function createQuoteProcessorContext(CartInterface $quote): ProcessorContextInterface
    {
        $context = $this->contextFactory->create([
            'context' => [
                static::QUOTE_CONTEXT => $quote
            ]
        ]);
        return $this->inheritContext($context);
    }

    /**
     * Create processor context for given order
     *
     * @param OrderInterface $order
     *
     * @return ProcessorContextInterface
     */
    public function createOrderProcessorContext(OrderInterface $order): ProcessorContextInterface
    {
        $context = $this->contextFactory->create([
            'context' => [
                static::ORDER_CONTEXT => $order
            ]
        ]);
        return $this->inheritContext($context);
    }

    /**
     * Create order extension from given syncspider order
     *
     * @param SyncSpiderOrderInterface $syncspiderOrder
     *
     * @return OrderExtensionInterface
     */
    public function createOrderExtension(SyncSpiderOrderInterface $syncspiderOrder): OrderExtensionInterface
    {
        $orderExtension = $syncspiderOrder->getExtensionAttributes();

        if (method_exists($orderExtension, 'setSyncspiderProcessorContext')) {
            $processorContext = $this->createProcessorContext($syncspiderOrder);
            $orderExtension->setSyncspiderProcessorContext($processorContext);
        }

        return $orderExtension;
    }

    /**
     * Get syncspider order from given context
     *
     * @param ProcessorContextInterface $context
     *
     * @return SyncSpiderOrderInterface|null
     */
    public function getSyncspiderOrder(ProcessorContextInterface $context): ?SyncSpiderOrderInterface
    {
        $order = $context->getContextValue(static::SYNCSPIDER_ORDER_CONTEXT);

        if ($order instanceof SyncSpiderOrderInterface) {
            return $order;
        }

        return null;
    }

    /**
     * Get quote from given context
     *
     * @param ProcessorContextInterface $context
     *
     * @return CartInterface|null
     */
    public function getQuote(ProcessorContextInterface $context): ?CartInterface
    {
        $quote = $context->getContextValue(static::QUOTE_CONTEXT);

        if ($quote instanceof CartInterface) {
            return $quote;
        }

        return null;
    }

    /**
     * Get order from given context
     *
     * @param ProcessorContextInterface $context
     *
     * @return OrderInterface|null
     */
    public function getOrder(ProcessorContextInterface $context): ?OrderInterface
    {
        $order = $context->getContextValue(static::ORDER_CONTEXT);

        if ($order instanceof OrderInterface) {
            return $order;
        }

        return null;
    }

    /**
     * Get order extension from given context
     *
     * Primarily retrieve from syncspider order context value,
     * secondarily retrieve from quote context value, thirdly
     * retrieve from order context value
     *
     * @param ProcessorContextInterface $context
     *
     * @return OrderExtensionInterface|null
     */
    public function getOrderExtension(ProcessorContextInterface $context): ?OrderExtensionInterface
    {
        $syncspiderOrder = $this->getSyncspiderOrder($context);

        if ($syncspiderOrder) {
            $extensionAttributes = $syncspiderOrder->getExtensionAttributes();

            if ($extensionAttributes instanceof OrderExtensionInterface) {
                return $extensionAttributes;
            }
        }

        $quote = $this->getQuote($context);

        if ($quote) {
            $extensionAttributes = $quote->getExtensionAttributes();

            if ($extensionAttributes && method_exists($extensionAttributes, 'getSyncspiderOrderExtension')) {
                $extensionAttributes = $extensionAttributes->getSyncspiderOrderExtension();

                if ($extensionAttributes instanceof OrderExtensionInterface) {
                    return $extensionAttributes;
                }
            }
        }

        $order = $this->getOrder($context);

        if ($order instanceof OrderInterface) {
            $extensionAttributes = $order->getExtensionAttributes();

            if ($extensionAttributes instanceof OrderExtensionInterface) {
                return $extensionAttributes;
            }
        }

        return null;
    }
}