<?php

namespace SyncSpider\APIextension\Model;

use Magento\Framework\Api\SearchCriteriaInterface;
use Magento\Framework\Api\Search\FilterGroup;
use Magento\Framework\Data\Collection\AbstractDb;
use SyncSpider\APIextension\Api\FilterMetadataInterface;
use SyncSpider\APIextension\Api\FilterRendererInterface;
use SyncSpider\APIextension\Api\FilterGroupProcessorInterface;
use Closure;
use Exception;

class FilterGroupProcessor implements FilterGroupProcessorInterface
{
    /**
     * @var FilterRendererInterface
     */
    private $filterRenderer;

    /**
     * @var FilterMetadataInterface
     */
    private $filterMetadata;

    /**
     * @var Closure
     */
    private $mergeBinds;

    /**
     * @var Closure
     */
    private $getBinds;

    /**
     * OrderBillingAddressProcessor constructor.
     *
     * @param FilterRendererInterface $filterRenderer
     * @param FilterMetadataInterface $filterMetadata
     */
    public function __construct(
        FilterRendererInterface $filterRenderer,
        FilterMetadataInterface $filterMetadata
    ) {
        $this->filterRenderer = $filterRenderer;
        $this->filterMetadata = $filterMetadata;
        $this->mergeBinds = function (array $binds) {
            if (property_exists($this, '_bindParams') && is_array($this->_bindParams)) {
                $this->_bindParams = array_merge($this->_bindParams, $binds);
            }
        };
        $this->getBinds = function () {
            if (property_exists($this, '_bindParams') && is_array($this->_bindParams)) {
                return $this->_bindParams;
            }

            return [];
        };
    }

    /**
     * @inheritDoc
     */
    public function handles(FilterGroup $filterGroup): bool
    {
        foreach ($filterGroup->getFilters() as $filter) {
            if (strpos($filter->getField(), $this->filterMetadata->getFieldPrefix()) !== 0) {
                return false;
            }
        }

        return true;
    }

    /**
     * @inheritDoc
     */
    public function apply(AbstractDb $collection, SearchCriteriaInterface $searchCriteria): AbstractDb
    {
        if (count($searchCriteria->getFilterGroups()) > 0) {
            $resource = $collection->getResource();
            $select = $collection->getConnection()->select()
                ->from([
                    $this->filterMetadata->getTableAlias() => $resource->getTable($this->filterMetadata->getTableName())
                ])
                ->where($this->filterMetadata->getConstraint());
            // apply suffix bind name so it can be used inside filter renderer to assign unique bind names
            $select->bind([
                FilterRendererInterface::SUFFIX_BIND_NAME => count($this->getBinds->call($collection))
            ]);

            foreach ($searchCriteria->getFilterGroups() as $group) {
                if (! $this->handles($group)) {
                    $groupJson = json_encode($group->__toArray());
                    throw new Exception(
                        'Filter group processor ' . static::class . ' does not handle given filter group: ' . $groupJson
                    );
                }

                $conditions = [];

                foreach ($group->getFilters() as $filter) {
                    $conditions[] = $this->filterRenderer->render($filter, $select, $this->filterMetadata);
                }

                $select->where(sprintf('%s', implode(' OR ', $conditions)));
            }

            $select->limit(1);
            $collection->getSelect()->where(sprintf('EXISTS(%s)', $select->assemble()));
            // apply binds to collection by removing the suffix bind name
            $binds = $select->getBind();
            unset($binds[FilterRendererInterface::SUFFIX_BIND_NAME]);
            $this->mergeBinds->call($collection, $binds);
        }

        return $collection;
    }
}
