<?php

namespace SyncSpider\APIextension\Model\Order;

use Exception;
use Magento\Framework\Exception\LocalizedException;
use Magento\Framework\Exception\NoSuchEntityException;

class Order
{
    /**
     * @var \Magento\Customer\Api\Data\CustomerInterfaceFactory
     */
    private $customerFactory;
    /**
     * @var \SyncSpider\APIextension\Model\Order\Quote
     */
    private $syncspiderQuote;
    /**
     * @var \Magento\Backend\Model\Session\Quote
     */
    private $sessionQuote;
    /**
     * @var \SyncSpider\APIextension\Api\ProductRepositoryInterface
     */
    private $productRepository;
    /**
     * @var \Magento\Customer\Api\CustomerRepositoryInterface
     */
    private $customerRepository;
    /**
     * @var \SyncSpider\APIextension\Helper\Data
     */
    private $helper;
    /**
     * @var \Magento\Catalog\Model\ProductFactory
     */
    private $productFactory;

    public function __construct(
        \Magento\Customer\Api\Data\CustomerInterfaceFactory $customerFactory,
        \SyncSpider\APIextension\Model\Order\Quote $syncspiderQuote,
        \Magento\Backend\Model\Session\Quote $sessionQuote,
        \Magento\Catalog\Api\ProductRepositoryInterface $productRepository,
        \Magento\Customer\Api\CustomerRepositoryInterface $customerRepository,
        \SyncSpider\APIextension\Helper\Data $helper,
        \Magento\Catalog\Model\ProductFactory $productFactory
    ) {
        $this->customerFactory = $customerFactory;
        $this->syncspiderQuote = $syncspiderQuote;
        $this->sessionQuote = $sessionQuote;
        $this->productRepository = $productRepository;
        $this->customerRepository = $customerRepository;
        $this->helper = $helper;
        $this->productFactory = $productFactory;
    }

    /**
     * Creates order
     *
     * @param \SyncSpider\APIextension\Api\Data\OrderInterface $order
     * @param \Magento\Store\Api\Data\StoreInterface $store
     *
     * @return \Magento\Sales\Model\Order
     * @throws Exception
     */
    public function create($order, $store)
    {
        $customer = $this->getCustomer($order, $store);
        $this->createProducts($order, $store);

        try {
            $quote = $this->syncspiderQuote->get($order, $customer, $store);
            $newOrder = $quote->createOrder();
            $this->sessionQuote->clearStorage();
            return $newOrder;
        } catch (Exception $e) {
            $this->sessionQuote->clearStorage();
            throw new \Magento\Framework\Exception\LocalizedException(
                __($e->getMessage())
            );
        }
    }

    /**
     * Get customer for given order and store
     *
     * @param \SyncSpider\APIextension\Api\Data\OrderInterface $order
     * @param \Magento\Store\Api\Data\StoreInterface $store $store
     *
     * @return \Magento\Customer\Api\Data\CustomerInterface|\Magento\Customer\Model\Customer|null
     *
     * @throws Exception
     */
    private function getCustomer($order, $store)
    {
        try {
            $customer = $this->customerRepository->get($order->getCustomerEmail(), $store->getWebsiteId());
        } catch (NoSuchEntityException $e) {
            $customer = null;
        } catch (LocalizedException $e) {
            $customer = null;
        }

        if (!$customer && $order->isGuestCustomer()) {
            return $customer;
        }

        if (!$customer && $order->shouldCreateCustomer()) {
            return $this->createCustomer($order->getCustomer(), $store);
        }

        return $customer;
    }

    /**
     * Create missing products for given order and store
     *
     * @param \SyncSpider\APIextension\Api\Data\OrderInterface $order
     * @param \Magento\Store\Api\Data\StoreInterface $store $store
     *
     * @throws Exception
     */
    private function createProducts($order, $store)
    {
        /** @var \SyncSpider\APIextension\Model\Data\OrderItem $product */
        foreach ($order->getProducts() as &$product) {
            $this->createProduct($product, $store);
        }
    }

    /**
     * Creates product used for created order
     *
     * @param \SyncSpider\APIextension\Model\Data\OrderItem $product
     * @param \Magento\Store\Api\Data\StoreInterface $store $store
     *
     * @throws Exception
     */
    private function createProduct($product, $store)
    {
        /** @var \Magento\Catalog\Model\Product $productModel */
        $productModel = $this->productFactory->create();

        if ($productModel->getIdBySku($product->getSku())) {
            return;
        }

        $productModel->setData($product->toArray());
        $productModel->setStoreId($store->getId());
        $productModel->setWebsiteIds([$store->getWebsiteId()]);
        $productModel->setTypeId(\Magento\Catalog\Model\Product\Type::TYPE_SIMPLE);
        $productModel->setVisibility(\Magento\Catalog\Model\Product\Visibility::VISIBILITY_NOT_VISIBLE);
        $productModel->setPrice($product['custom_price']);
        $productModel->setStatus(true);
        $productModel->setStockData(
            [
                'use_config_manage_stock' => 0,
                'manage_stock' => 1,
                'is_in_stock' => 1,
                'backorders' => 1,
                'use_config_backorders' => 0,
                'qty' => $product->getQty()
            ]
        );

        try {
            if (version_compare($this->helper->getVersion(), '2.1.0', '>')) {
                $this->productRepository->save($productModel);
            } else {
                $productModel->save();
            }
        } catch (Exception $e) {
            throw $e;
        }
    }

    /**
     * Creates customer
     *
     * @param \SyncSpider\APIextension\Api\Data\OrderCustomerInterface $orderCustomer
     * @param \Magento\Store\Api\Data\StoreInterface $store $store
     *
     * @return \Magento\Customer\Api\Data\CustomerInterface
     * @throws Exception
     */
    private function createCustomer($orderCustomer, $store)
    {
        $customer = $this->customerFactory->create();
        $customer->setWebsiteId($store->getWebsiteId());
        $customer->setEmail($orderCustomer->getEmail());
        $customer->setFirstname($orderCustomer->getFirstname());
        $customer->setLastname($orderCustomer->getLastname());
        $customer->setGroupId($orderCustomer->getGroupId());

        try {
            $this->customerRepository->save($customer);
            return $this->customerRepository->get($customer->getEmail(), $store->getWebsiteId());
        } catch (Exception $e) {
            throw $e;
        }
    }
}
