<?php

namespace SyncSpider\APIextension\Model\Order;

use Exception;
use Magento\Tax\Model\Config;
use Magento\Framework\Exception\LocalizedException;
use Magento\Sales\Model\Order\Email\Container\OrderIdentity;
use SyncSpider\APIextension\Api\Data\OrderPaymentOptionInterface;
use SyncSpider\APIextension\Shipping\Carrier\TransferObject;
use Magento\Framework\App\ObjectManager;
use Magento\Store\Api\Data\StoreInterface;
use Magento\Customer\Api\Data\CustomerInterface;
use Magento\Customer\Model\Customer;
use Magento\Quote\Model\Quote as MagentoQuote;
use SyncSpider\APIextension\Api\Data\OrderInterface as SyncSpiderOrderInterface;
use Magento\Framework\Stdlib\DateTime\TimezoneInterface;
use SyncSpider\APIextension\Api\Data\ProductOptionInterface;
use SyncSpider\APIextension\Model\{
    ProductOption\ProductOptionProcessor,
    ExtensionAttributes\SalesOrderContext
};

class Quote
{
    /**
     * @var \SyncSpider\APIextension\Api\Data\OrderInterface
     */
    private $order = null;
    /**
     * @var \Magento\Store\Model\Store
     */
    private $store = null;
    /**
     * @var Customer|CustomerInterface
     */
    private $customer = null;
    /**
     * @var \Magento\Directory\Model\Currency
     */
    private $currencyDirectory;
    /**
     * @var \Magento\Sales\Model\AdminOrder\Create
     */
    private $salesOrderCreate;
    /**
     * @var \Magento\Tax\Model\Config
     */
    private $taxConfig;
    /**
     * @var \Magento\Backend\Model\Session\Quote
     */
    private $sessionQuote;
    /**
     * @var \SyncSpider\APIextension\Model\Order\Configuration
     */
    private $configuration;
    /**
     * @var \Magento\Framework\Locale\FormatInterface
     */
    private $localeFormat;
    /**
     * @var \Magento\Catalog\Api\ProductRepositoryInterface
     */
    private $productRepository;
    /**
     * @var \Magento\Quote\Api\CartRepositoryInterface
     */
    private $cartRepositoryInterface;

    /**
     * @var TimezoneInterface
     */
    private $timezone;

    /**
     * @var ProductOptionProcessor
     */
    private $productOptionProcessor;

    /**
     * @var SalesOrderContext
     */
    private $orderContext;

    public function __construct(
        \Magento\Directory\Model\Currency $currencyDirectory,
        \Magento\Sales\Model\AdminOrder\Create $salesOrderCreate,
        \Magento\Tax\Model\Config $taxConfig,
        \Magento\Backend\Model\Session\Quote $sessionQuote,
        \SyncSpider\APIextension\Model\Order\Configuration $configuration,
        \Magento\Catalog\Api\ProductRepositoryInterface $productRepository,
        \Magento\Framework\Locale\FormatInterface $localeFormat,
        \Magento\Quote\Api\CartRepositoryInterface $cartRepositoryInterface,
        TimezoneInterface $timezone,
        ProductOptionProcessor $productOptionProcessor,
        SalesOrderContext $orderContext
    ) {
        $this->currencyDirectory = $currencyDirectory;
        $this->salesOrderCreate = $salesOrderCreate;
        $this->taxConfig = $taxConfig;
        $this->sessionQuote = $sessionQuote;
        $this->configuration = $configuration;
        $this->localeFormat = $localeFormat;
        $this->productRepository = $productRepository;
        $this->cartRepositoryInterface = $cartRepositoryInterface;
        $this->timezone = $timezone;
        $this->productOptionProcessor = $productOptionProcessor;
        $this->orderContext = $orderContext;
    }

    /**
     * Gets prepared quote for provided order, customer and store
     *
     * @param \SyncSpider\APIextension\Api\Data\OrderInterface $order
     * @param \Magento\Customer\Model\Customer|null $customer
     * @param \Magento\Store\Api\Data\StoreInterface $store
     *
     * @return \Magento\Sales\Model\AdminOrder\Create
     * @throws Exception
     */
    public function get($order, $customer, $store)
    {
        $this->order = $order;
        $this->customer = $customer;
        $this->store = $store;

        try {
            $this->initStoreConfig();
            $this->initCustomer();
            $this->initAddresses();
            $this->initCurrency();
            $this->initQuoteItems();
            $this->initShippingInfo();
            $this->initPaymentInfo();

            $this->salesOrderCreate->importPostData($this->order->toArray());
            $this->salesOrderCreate->collectShippingRates();
            $this->salesOrderCreate->initRuleData();

            $this->processQuote($this->getQuote(), $order, $customer, $store);
            $this->cartRepositoryInterface->save($this->getQuote()->collectTotals());
            /** @var \Magento\Quote\Model\Quote $createdQuote */
            $createdQuote = $this->cartRepositoryInterface->get($this->getQuote()->getId());
            $createdQuote->setCustomer($this->getQuote()->getCustomer());
            $createdQuote = $this->recalculateSavedQuote($createdQuote);

            // assign syncspider order extension attributes to quote cart "syncspider_order_extension" extension attribute
            if ($createdQuote->getExtensionAttributes()) {
                if (method_exists($createdQuote->getExtensionAttributes(), 'setSyncspiderOrderExtension')) {
                    $createdQuote->getExtensionAttributes()
                        ->setSyncspiderOrderExtension(
                            $this->orderContext->createOrderExtension($order)
                        );
                }
            }
        } catch (Exception $e) {
            throw $e;
        }

        return $this->salesOrderCreate->setQuote($createdQuote);
    }

    /**
     * Reset store configs to initial state
     *
     * @void
     */
    public function __destruct()
    {
        /** @var \SyncSpider\APIextension\Model\Order\Configuration $configuration */
        $configuration = ObjectManager::getInstance()->get(\SyncSpider\APIextension\Model\Order\Configuration::class);
        $configuration->clear();
    }

    /**
     * Recalculates quote saved in database.
     *
     * Must be performed again because of known bug in quote
     * getItemById https://github.com/magento/magento2/commit/1fb3a97b4f394ae05d7afc5a213d0eb22d6e7c2e
     *
     * @param \Magento\Quote\Model\Quote $createdQuote
     *
     * @return \Magento\Quote\Model\Quote
     */
    private function recalculateSavedQuote($createdQuote)
    {
        /** @var \SyncSpider\APIextension\Api\Data\OrderItemInterface $product */
        foreach ($this->order->getProducts() as $product) {
            /** @var \Magento\Quote\Model\ResourceModel\Quote\Item $item */
            foreach ($createdQuote->getAllItems() as $item) {
                if ($product->getTaxClassId() !== null && $item->getProduct()->getSku() === $product->getSku()) {
                    $item->getProduct()->setTaxClassId($product->getTaxClassId());
                }
            }
        }

        return $createdQuote->collectTotals();
    }

    /**
     * Initializes payment information that will be shown on order
     *
     * @return void
     */
    private function initPaymentInfo()
    {
        $payment = $this->getQuote()->getPayment();

        /** @var \SyncSpider\APIextension\Model\Data\OrderPayment $paymentInfo */
        if ($paymentInfo = $this->order->getPayment()) {
            $paymentData = $paymentInfo->toArray();
            if ($this->order->getPayment()->getPaymentAdditional()) {
                /** @var \SyncSpider\APIextension\Api\Data\OrderPaymentAdditionalInterface $info */
                foreach ($this->order->getPayment()->getPaymentAdditional() as $info) {
                    $paymentData[$info->getKey()] = $info->getValue();
                }
            }

            $payment->setData(OrderPaymentOptionInterface::ATTRIBUTE_CODE, $paymentData);
        }

        $payment->setMethod(\SyncSpider\APIextension\Api\PaymentInterface::CODE);
    }

    /**
     * Initializes shipping info
     *
     * @throws Exception
     */
    private function initShippingInfo()
    {
        if ($shipping = $this->order->getShipping()) {
            if ($this->store->getCurrentCurrency() && $this->store->getBaseCurrency() && $shipping->getShippingCost()) {
                $price = $this->store->getCurrentCurrency()->convert(
                    $shipping->getShippingCost(),
                    $this->store->getBaseCurrency()
                );
                $this->order->getShipping()->setShippingCost($price);
            }
        }

        /** @var \SyncSpider\APIextension\Shipping\Carrier\TransferObject $transferObject */
        $transferObject = ObjectManager::getInstance()->get(TransferObject::class);
        $transferObject->setIsUsed(true)->setShipping($this->order->getShipping());
        $this->getQuote()->getShippingAddress()->setShippingMethod('syncspider_syncspider');
    }

    /**
     * Initializes quote items, expected structure for each item:
     *
     * [
     *  [
     *      'id' => 1,
     *      'qty' => 1,
     *      'custom_price' => 1,
     *      'use_discount' => true
     *  ]
     * ]
     *
     * @throws Exception
     */
    private function initQuoteItems()
    {
        /** @var \SyncSpider\APIextension\Api\Data\OrderItemInterface $orderItem */
        foreach ($this->order->getProducts() as $orderItem) {
            /** @var \Magento\Catalog\Model\Product $product */
            $product = $this->productRepository->get($orderItem->getSku(), false, $this->store->getId());

            if ($orderItem->getTaxClassId() !== null) {
                $product->setData('tax_class_id', $orderItem->getTaxClassId());
            }

            $config['qty'] = $orderItem->getOrderedQty() ? (float)$orderItem->getOrderedQty() : 1;

            // attach buy request data based on item product option
            if ($orderItem->getProductOption()) {
                $buyRequest = $this->productOptionProcessor->convertToBuyRequest($orderItem->getProductOption());
                $config = array_merge($config, $buyRequest->getData());
            }

            $this->salesOrderCreate->addProduct($product, $config);
            /** @var \Magento\Quote\Model\Quote\Item $item */
            $item = $this->getQuote()->getItemByProduct($product);
            if ($orderItem->getCustomPrice() !== null) {
                $price = $this->localeFormat->getNumber($orderItem->getCustomPrice());
                $price = $price > 0 ? $price : 0;
                $item->setCustomPrice($price);
                $item->setOriginalCustomPrice($price);
                $item->setNoDiscount($orderItem->isUseDiscount() === true);
            }

            // set gift messages on product level
            if ($orderItem->getGiftMessage() && $orderItem->getGiftMessage()->getMessage()) {
                /** @var \Magento\GiftMessage\Model\Message $giftMessage */
                $giftMessage = ObjectManager::getInstance()->create(\Magento\GiftMessage\Model\Message::class);
                $giftMessage->setSender($orderItem->getGiftMessage()->getSender());
                $giftMessage->setRecipient($orderItem->getGiftMessage()->getRecipient());
                $giftMessage->setMessage($orderItem->getGiftMessage()->getMessage());

                if (!$giftMessage->isMessageEmpty()) {
                    $giftMessage = $giftMessage->save();
                    $item->setGiftMessageId($giftMessage->getId());
                }
            }
        }
    }

    /**
     * Initializes customer. If customer is not provided, it should be treated as guest customer
     *
     * @return void
     */
    private function initCustomer()
    {
        // customer is initialized and exists, otherwise treat customer as guest
        if ($this->customer) {
            $customerGroupId = $this->order->getCustomerGroupId() !== null ?
                $this->order->getCustomerGroupId() : $this->customer->getGroupId();

            $this->getQuote()->assignCustomer($this->customer);

            $this->order->setAccount(
                [
                    CustomerInterface::EMAIL => $this->customer->getEmail(),
                    CustomerInterface::GROUP_ID => $customerGroupId,
                ]
            );
            $this->sessionQuote->setCustomerId((int)$this->customer->getId());
        } else {
            $customer = $this->order->getCustomer();
            $customerGroupId = $this->order->getCustomerGroupId() !== null ?
                $this->order->getCustomerGroupId() : \Magento\Customer\Model\GroupManagement::NOT_LOGGED_IN_ID;
            // assign data to quote customer
            $quoteCustomer = $this->getQuote()->getCustomer();
            $quoteCustomer->setEmail($customer->getEmail())
                ->setFirstname($customer->getFirstname())
                ->setLastname($customer->getLastname())
                ->setGroupId($customerGroupId)
                ->setPrefix($customer->getPrefix())
                ->setDob($customer->getDob());
            // assign quote customer data to quote data
            $quote = $this->getQuote();
            $quote->setCustomerId(null)
                ->setStore($this->store)
                ->setCustomerEmail($quoteCustomer->getEmail())
                ->setCustomerFirstname($quoteCustomer->getFirstname())
                ->setCustomerLastname($quoteCustomer->getLastname())
                ->setCustomerGroupId($quoteCustomer->getGroupId())
                ->setCustomerIsGuest(true)
                ->setCustomerPrefix($quoteCustomer->getPrefix())
                ->setCustomerDob($quoteCustomer->getDob());

            $this->order->setAccount(
                [
                    CustomerInterface::EMAIL => $customer->getEmail(),
                    CustomerInterface::GROUP_ID => $customerGroupId,
                    CustomerInterface::FIRSTNAME => $customer->getFirstname(),
                    CustomerInterface::LASTNAME => $customer->getLastname(),
                    CustomerInterface::PREFIX => $customer->getPrefix(),
                    CustomerInterface::DOB => $customer->getDob()? $this->timezone->formatDate($customer->getDob()) : null
                ]
            );
            $this->sessionQuote->setCustomerId($this->order->getCustomerEmail());
        }
    }

    /**
     * Initializes shipping and billing addresses
     */
    private function initAddresses()
    {
        if ($sameAsBilling = $this->order->isSameAsBilling()) {
            $this->order->setShippingAddress(null);
            $this->salesOrderCreate->setShippingAsBilling($sameAsBilling);
        }
    }

    /**
     * Initializes currency for store
     *
     * @return void
     */
    private function initCurrency()
    {
        $currentCurrency = $this->currencyDirectory->load($this->order->getCurrencyCode());
        $this->getQuote()->setCurrency();
        $this->store->setCurrentCurrencyCode($currentCurrency->getCode());
    }

    /**
     * Initializes store config and task calculation
     */
    private function initStoreConfig()
    {
        $storeCode = $this->store->getCode();
        $objectManager = \Magento\Framework\App\ObjectManager::getInstance();
        /** @var \SyncSpider\APIextension\Model\Order\Configuration $configuration */
        $configuration = $objectManager->get(\SyncSpider\APIextension\Model\Order\Configuration::class);

        if ($this->order->isPriceIncludeTax() !== null) {
            $configuration->setValue(
                Config::CONFIG_XML_PATH_PRICE_INCLUDES_TAX,
                $this->order->isPriceIncludeTax(),
                $storeCode
            );

            $configuration->setValue(
                Config::CONFIG_XML_PATH_SHIPPING_INCLUDES_TAX,
                $this->order->isPriceIncludeTax(),
                $storeCode
            );

            // when catalog price includes tax, enable this setting will
            // fix the price no matter what the customer's tax rate is.
            if ($this->order->isPriceIncludeTax()) {
                $configuration->setValue(
                    Config::CONFIG_XML_PATH_CROSS_BORDER_TRADE_ENABLED,
                    true,
                    $storeCode
                );
            }
        }

        if ($this->order->getShippingTaxClassId() !== null) {
            $configuration->setValue(
                Config::CONFIG_XML_PATH_SHIPPING_TAX_CLASS,
                $this->order->getShippingTaxClassId(),
                $storeCode
            );
        }

        $configuration->setValue(OrderIdentity::XML_PATH_EMAIL_ENABLED, false, $storeCode);

        // Must be set, let Magento core to perform all validations, such as
        // quantity check for product. Otherwise, Magento skips all validations and creates order
        $this->getQuote()->setIsSuperMode(false);
        $this->getQuote()->setStore($this->store);
        $this->getQuote()->setStoreId($this->store->getId());
        $this->sessionQuote->setStore($this->store);
        $this->sessionQuote->setStoreId($this->store->getId());
    }

    /**
     * Retrieves current quote model
     *
     * @return \Magento\Quote\Model\Quote
     */
    private function getQuote()
    {
        return $this->salesOrderCreate->getQuote();
    }

    /**
     * Allow modifing given quote based on given syncspider order, customer and store
     *
     * @param MagentoQuote $quote
     * @param SyncSpiderOrderInterface $syncspiderOrder
     * @param Customer|null $customer
     * @param StoreInterface $store
     *
     * @return void
     */
    public function processQuote(MagentoQuote $quote, SyncSpiderOrderInterface $syncspiderOrder, Customer $customer = null, StoreInterface $store)
    {
        return $this;
    }
}
