<?php

namespace SyncSpider\APIextension\Model\Order;

use Magento\Sales\Api\{
    Data\OrderInterface,
    OrderRepositoryInterface
};
use Magento\Framework\Exception\NoSuchEntityException;
use Magento\Framework\Api\SearchCriteriaBuilderFactory;
use Magento\Framework\Exception\LocalizedException;
use SyncSpider\APIextension\Api\Data\OrderInterface as SyncSpiderOrderInterface;
use Exception;

class Validation
{
    /**
     * Required fields
     *
     * @var array
     */
    private static $requiredGlobal = [
        'products',
        'customer',
    ];

    /**
     * Required fields when customer should be created dynamically
     *
     * @var array
     */
    private static $requiredGuestCustomer = [
        'email',
        'firstname',
        'lastname',
    ];

    /**
     * Required fields when order items are set
     *
     * @var array
     */
    private static $requiredOrderItems = [
        'sku',
        'ordered_qty',
    ];

    /**
     * Required fields when order items are created dynamically
     *
     * @var array
     */
    private static $requiredProductCreation = [
        'name',
        'attribute_set_id',
        'custom_price',
        'tax_class_id',
    ];

    /**
     * Required fields when customer is created dynamically
     *
     * @var array
     */
    private static $requiredCustomerCreation = [
        'email',
        'firstname',
        'lastname',
        'group_id',
    ];

    /**
     * @var SyncSpiderOrderInterface
     */
    private $order = null;

    /**
     * @var \Magento\Store\Model\Store
     */
    private $store = null;

    /**
     * @var \Magento\Store\Api\Data\StoreInterface
     */
    private $helper;

    /**
     * @var \Magento\Eav\Api\AttributeSetRepositoryInterface
     */
    private $attributeSet;

    /**
     * @var \Magento\Tax\Api\TaxClassRepositoryInterface
     */
    private $taxClassRepository;

    /**
     * @var \Magento\Customer\Model\CustomerFactory
     */
    private $customerFactory;

    /**
     * @var \Magento\Catalog\Model\ProductFactory
     */
    private $productFactory;

    /**
     * @var OrderRepositoryInterface
     */
    private $orderRepository;

    /**
     * @var SearchCriteriaBuilderFactory
     */
    private $searchCriteriaBuilderFactory;

    /**
     * Validation constructor.
     *
     * @param \Magento\Eav\Api\AttributeSetRepositoryInterface $attributeSet
     * @param \Magento\Tax\Api\TaxClassRepositoryInterface $taxClassRepository
     * @param \Magento\Customer\Model\CustomerFactory $customerFactory
     * @param \Magento\Catalog\Model\ProductFactory $productFactory
     * @param \SyncSpider\APIextension\Helper\Data $helper
     * @param OrderRepositoryInterface $orderRepository
     * @param SearchCriteriaBuilderFactory $searchCriteriaBuilderFactory
     */
    public function __construct(
        \Magento\Eav\Api\AttributeSetRepositoryInterface $attributeSet,
        \Magento\Tax\Api\TaxClassRepositoryInterface $taxClassRepository,
        \Magento\Customer\Model\CustomerFactory $customerFactory,
        \Magento\Catalog\Model\ProductFactory $productFactory,
        \SyncSpider\APIextension\Helper\Data $helper,
        OrderRepositoryInterface $orderRepository,
        SearchCriteriaBuilderFactory $searchCriteriaBuilderFactory
    ) {
        $this->attributeSet = $attributeSet;
        $this->taxClassRepository = $taxClassRepository;
        $this->customerFactory = $customerFactory;
        $this->productFactory = $productFactory;
        $this->helper = $helper;
        $this->orderRepository = $orderRepository;
        $this->searchCriteriaBuilderFactory = $searchCriteriaBuilderFactory;
    }

    /**
     * Validate given order in given store
     *
     * @param \SyncSpider\APIextension\Api\Data\OrderInterface $order
     * @param \Magento\Store\Api\Data\StoreInterface $store
     *
     * @return void
     *
     * @throws Exception
     */
    public function validate($order, $store)
    {
        $this->order = $order;
        $this->store = $store;

        $this->validateRequired($this->order->toArray(), self::$requiredGlobal);
        $this->validateCustomer();
        $this->validateCurrency();
        $this->validateOrderItems();
        $this->validateOrder($order);
    }

    /**
     * Validates if currency is allowed
     *
     * @return void
     *
     * @throws Exception
     */
    private function validateCurrency()
    {
        $allowedCurrencies = $this->store->getAvailableCurrencyCodes();
        $baseCurrency = $this->store->getBaseCurrency();

        $currencyCode = $this->order->getCurrencyCode() ?
            $this->order->getCurrencyCode() : $this->store->getBaseCurrency()->getCode();

        if ($currencyCode && !in_array($currencyCode, $allowedCurrencies)) {
            throw new LocalizedException(
                __('Currency code "%1" is not allowed for store "%2".', [$currencyCode, $this->store->getCode()])
            );
        }

        if ($currencyCode !== $baseCurrency &&
            $this->helper->getConvertRateFromBase($currencyCode, $this->store) == 0
        ) {
            throw new LocalizedException(
                __('Provided currency code "%1" is convertible to base currency "%2".', [$currencyCode, $baseCurrency])
            );
        }
    }

    /**
     * Validate current order customer
     *
     * @return void
     *
     * @throws Exception
     */
    private function validateCustomer()
    {
        /** @var \Magento\Customer\Model\Customer $model */
        $model = $this->customerFactory->create();

        $customerEmail = $this->order->getCustomerEmail();
        $isGuest = $this->order->isGuestCustomer();
        $createCustomer = $this->order->isCreateCustomer();

        if (!$isGuest && !$customerEmail && !$createCustomer) {
            throw new LocalizedException(
                __('Customer is not defined. Skipping order import.')
            );
        }

        $customer = $model->setWebsiteId($this->store->getWebsiteId())->loadByEmail($customerEmail);
        /** @var \SyncSpider\APIextension\Model\Data\OrderCustomer $orderCustomer */
        $orderCustomer = $this->order->getCustomer();

        if (!$customer->getId() && $isGuest) {
            $this->validateRequired($orderCustomer->toArray(), self::$requiredGuestCustomer);
        } elseif (!$customer->getId() && !$isGuest && $createCustomer) {
            $this->validateRequired($orderCustomer->toArray(), self::$requiredCustomerCreation);
        } elseif (!$customer->getId()) {
            throw new LocalizedException(
                __('Customer with email "%1" is not found. Skipping order import.)', [$customerEmail])
            );
        }
    }

    /**
     * Whether an magento 2 order already exists with given incremental id
     *
     * @param SyncSpiderOrderInterface $order
     *
     * @return Validation
     *
     * @throws LocalizedException
     */
    private function validateIncrementalId(SyncSpiderOrderInterface $order)
    {
        if ($order->getIncrementId()) {
            $searchCriteriaBuilder = $this->searchCriteriaBuilderFactory->create()
                ->addFilter(OrderInterface::INCREMENT_ID, $order->getIncrementId())
                ->setPageSize(1);
            $list = $this->orderRepository->getList($searchCriteriaBuilder->create());

            if (count($list->getItems()) > 0) {
                throw new LocalizedException(
                    __(
                        'Order already exists with provided increment ID "%1"',
                        [$order->getIncrementId()]
                    )
                );
            }
        }

        return $this;
    }

    /**
     * Whether an magento 2 order already exists with given incremental id
     *
     * @param SyncSpiderOrderInterface $order
     *
     * @return Validation
     *
     * @throws LocalizedException
     */
    private function validateExtOrderId(SyncSpiderOrderInterface $order)
    {
        if ($order->getExtOrderId()) {
            $searchCriteriaBuilder = $this->searchCriteriaBuilderFactory->create()
                ->addFilter(OrderInterface::EXT_ORDER_ID, $order->getExtOrderId())
                ->setPageSize(1);
            $list = $this->orderRepository->getList($searchCriteriaBuilder->create());

            if (count($list->getItems()) > 0) {
                throw new LocalizedException(
                    __(
                        'Order already exists with provided external order ID "%1"',
                        [$order->getExtOrderId()]
                    )
                );
            }
        }

        return $this;
    }

    /**
     * Checks if order already exists
     *
     * @param SyncSpiderOrderInterface $order
     *
     * @return void
     *
     * @throws Exception
     */
    private function validateOrder(SyncSpiderOrderInterface $order)
    {
        if (empty($order->getIncrementId()) && empty($order->getExtOrderId())) {
            throw new LocalizedException(
                __("The 'Incremental ID' or 'External Order ID' field is required.")
            );
        }

        $this->validateIncrementalId($order)
            ->validateExtOrderId($order);

        if ($order->getShippingTaxClassId() !== null) {
            $this->validateTaxClass($order->getShippingTaxClassId());
        }
    }

    /**
     * Validate current order products
     *
     * @return void
     *
     * @throws Exception
     */
    private function validateOrderItems()
    {
        /** @var \SyncSpider\APIextension\Model\Data\OrderItem $product */
        foreach ($this->order->getProducts() as $product) {
            $this->validateRequired($product->toArray(), self::$requiredOrderItems);
            $productId = $this->productFactory->create()->getIdBySku($product->getSku());

            if (!$productId && !$this->order->shouldCreateProduct()) {
                throw new LocalizedException(
                    __('Product with SKU "%1" is not found.', [$product->getSku()])
                );
            }

            if (!$productId && $this->order->shouldCreateProduct()) {
                $this->validateRequired($product->toArray(), self::$requiredProductCreation);
                $this->validateAttributeSet($product->getAttributeSetId(), $product->getSku());
            }

            if ($product->getTaxClassId() !== null) {
                $this->validateTaxClass($product->getTaxClassId());
            }
        }
    }

    /**
     * Validate given tax class
     *
     * @param int $taxClassId
     *
     * @throws Exception
     */
    private function validateTaxClass($taxClassId)
    {
        // tax class "None" is hardcoded to zero
        if (0 === (int)$taxClassId) {
            return;
        }

        /** @var \Magento\Tax\Model\ClassModel $taxClass */
        try {
            $taxClass = $this->taxClassRepository->get($taxClassId);
        } catch (NoSuchEntityException $e) {
            throw new LocalizedException(
                __('Provided tax class "%1" not found.', [$taxClassId])
            );
        }

        if ($taxClass->getClassType() !== \Magento\Tax\Model\ClassModel::TAX_CLASS_TYPE_PRODUCT) {
            throw new LocalizedException(
                __('Provided tax class "%1" not found.', [$taxClassId])
            );
        }
    }

    /**
     * Validate given attribute set for given product sku
     *
     * @param int $attributeSetId
     * @param string $sku
     *
     * @throws Exception
     */
    private function validateAttributeSet($attributeSetId, $sku)
    {
        /** @var \Magento\Eav\Model\Entity\Attribute\Set $attributeSet */
        try {
            $this->attributeSet->get($attributeSetId);
        } catch (NoSuchEntityException $e) {
            throw new LocalizedException(
                __('Provided attribute set "%1" not found for product "%2".', [$attributeSetId, $sku])
            );
        }
    }

    /**
     * Validates required fields
     *
     * @param array $data
     * @param array $required
     *
     * @throws Exception
     */
    private function validateRequired($data, $required)
    {
        foreach ($required as $field) {
            if (!isset($data[$field])) {
                throw new LocalizedException(
                    __("Required field (%1) is missing.", [$field])
                );
            }
        }
    }
}
