<?php

namespace SyncSpider\APIextension\Model;

use Magento\Framework\App\ObjectManager;
use Magento\Framework\Api\SearchCriteriaBuilder;
use Magento\Catalog\Model\Product as MagentoProduct;
use Magento\Catalog\Pricing\Price\FinalPrice;
use Magento\Catalog\Pricing\Price\RegularPrice;
use Magento\ConfigurableProduct\Model\Product\Type\Configurable;
use SyncSpider\APIextension\Api\Data\ProductInterface;
use SyncSpider\APIextension\Api\Data\ProductParentInterface;
use SyncSpider\APIextension\Api\ProductRepositoryInterface;

class Product extends MagentoProduct implements ProductInterface
{
    /**
     * @var bool
     */
    private $isParentLoaded = false;

    /**
     * @var \Magento\Catalog\Model\Product
     */
    private $parent = null;

    /**
     * @inheritDoc
     */
    public function getParents()
    {
        $parents = [];
        $configurableType = ObjectManager::getInstance()->get(Configurable::class);
        $parentIds = $configurableType->getParentIdsByChild($this->getId());

        if (empty($parentIds)) {
            return $parents;
        }

        $productRepo = ObjectManager::getInstance()->get(ProductRepositoryInterface::class);
        $searchCriteriaBuilder = ObjectManager::getInstance()->get(SearchCriteriaBuilder::class);
        $searchCriteria = $searchCriteriaBuilder->addFilter('entity_id', $parentIds, 'in')->create();

        foreach ($productRepo->getList($searchCriteria)->getItems() as $parentItem) {
            $productParent = ObjectManager::getInstance()->create(ProductParentInterface::class);
            $parents[] = $productParent->setSku($parentItem->getSku());
        }

        return $parents;
    }

    /**
     * @inheritDoc
     */
    public function getStoreCodes()
    {
        if (! $this->hasStoreCodes()) {
            $storeCodes = [];

            if ($websiteIds = $this->getWebsiteIds()) {
                foreach ($websiteIds as $websiteId) {
                    $websiteStores = $this->_storeManager->getWebsite($websiteId)->getStoreCodes();
                    $storeCodes = array_merge($storeCodes, $websiteStores);
                }
            }

            $this->setStoreCodes($storeCodes);
        }

        return $this->getData('store_codes');
    }

    /**
     * @inheritDoc
     */
    public function getPriceInclTax()
    {
        return $this->getPriceInfo()
            ->getPrice(RegularPrice::PRICE_CODE)
            ->getAmount()
            ->getValue();
    }

    /**
     * @inheritDoc
     */
    public function getPriceExclTax()
    {
        return $this->getPriceInfo()
            ->getPrice(RegularPrice::PRICE_CODE)
            ->getAmount()
            ->getBaseAmount();
    }

    /**
     * @inheritDoc
     */
    public function getFinalPriceInclTax()
    {
        return $this->getPriceInfo()
            ->getPrice(FinalPrice::PRICE_CODE)
            ->getAmount()
            ->getValue();
    }

    /**
     * @inheritDoc
     */
    public function getFinalPriceExclTax()
    {
        return $this->getPriceInfo()
            ->getPrice(FinalPrice::PRICE_CODE)
            ->getAmount()
            ->getBaseAmount();
    }

    /**
     * @inheritDoc
     */
    public function getImages()
    {
        $result = $this->getMediaGalleryEntries();
        return !empty($result) ? $result : $this->returnParentImages();
    }

    /**
     * @inheritDoc
     */
    public function getVisibleProductUrl()
    {
        // if has parent and is not visible on site, use parent url
        $useParentUrl = $this->getFirstParent() && !$this->isVisibleInSiteVisibility();
        return $useParentUrl ? $this->parent->getProductUrl() : $this->getProductUrl();
    }

    /**
     * @inheritDoc
     */
    public function getCategories()
    {
        $result = $this->getCategoryIds();
        if (empty($result) && $this->getFirstParent()) {
            $result = $this->parent->getCategoryIds();
        }

        return $result;
    }

    /**
     * Get parent product gallery images
     *
     * @return \Magento\Catalog\Api\Data\ProductAttributeMediaGalleryEntryInterface[]|null
     */
    private function returnParentImages()
    {
        if (! $this->getFirstParent()) {
            return [];
        }

        return $this->parent->getMediaGalleryEntries();
    }

    /**
     * Get first available parent product
     *
     * @return ProductInterface
     */
    private function getFirstParent()
    {
        if (! $this->isParentLoaded) {
            $configurableType = ObjectManager::getInstance()->get(Configurable::class);
            $productRepo = ObjectManager::getInstance()->get(ProductRepositoryInterface::class);
            $searchCriteriaBuilder = ObjectManager::getInstance()->get(SearchCriteriaBuilder::class);
            $searchCriteria = $searchCriteriaBuilder
                ->addFilter('entity_id', $configurableType->getParentIdsByChild($this->getId()), 'in')
                ->create();
            $parents = $productRepo->getList($searchCriteria)->getItems();
            $this->parent = !empty($parents) ? reset($parents) : null;
            $this->isParentLoaded = true;
        }

        return $this->parent;
    }
}
