<?php

namespace SyncSpider\APIextension\Model;

use Magento\Catalog\Api\ProductRepositoryInterface;
use Magento\Framework\App\Filesystem\DirectoryList;
use Magento\Framework\ObjectManagerInterface;
use Magento\Framework\Webapi\Exception as WebapiException;
use Magento\Framework\Webapi\Rest\Request;
use Magento\Store\Model\StoreManagerInterface;
use SyncSpider\APIextension\Api\{
    ExtensionAttributes\ProcessorContextInterfaceFactory,
    ProductImagesInterface
};
use SyncSpider\APIextension\Model\Api\Response;
use SyncSpider\APIextension\Model\{
    ProductMediaGallery\EntriesResolverContextInterface,
    ProductMediaGallery\EntriesResolverContextInterfaceFactory,
    ProductMediaGalleryProcessor
};
use Exception;

class ProductImages implements ProductImagesInterface
{
    /**
     * @var ObjectManagerInterface
     */
    private $objectManager;

    /**
     * @var Request
     */
    private $request;
    /**
     * @var ProductRepositoryInterface
     */
    private $productRepository;

    /**
     * @var StoreManagerInterface
     */
    private $storeManager;

    /**
     * @var ProductMediaGalleryProcessor
     */
    private $productMediaGalleryProcessor;

    /**
     * @var EntriesResolverContextInterfaceFactory
     */
    private $entriesResolverContextFactory;

    /**
     * @var ProcessorContextInterfaceFactory
     */
    private $processorContextFactory;

    public function __construct(
        ObjectManagerInterface $objectManager,
        Request $request,
        ProductRepositoryInterface $productRepository,
        DirectoryList $directoryList,
        StoreManagerInterface $storeManager,
        ProductMediaGalleryProcessor $productMediaGalleryProcessor,
        EntriesResolverContextInterfaceFactory $entriesResolverContextFactory,
        ProcessorContextInterfaceFactory $processorContextFactory
    ) {
        $this->objectManager = $objectManager;
        $this->request = $request;
        $this->productRepository = $productRepository;
        $this->directoryList = $directoryList;
        $this->storeManager = $storeManager;
        $this->productMediaGalleryProcessor = $productMediaGalleryProcessor;
        $this->entriesResolverContextFactory = $entriesResolverContextFactory;
        $this->processorContextFactory = $processorContextFactory;
    }

    /**
     * @inheritDoc
     */
    public function execute()
    {
        /**
         * @var Response $response
         */
        $response = $this->objectManager->create(Response::class);
        $response->setSuccess(true);
        $response->setMessage('OK');

        $data = $this->request->getRequestData();
        $productId = isset($data['id']) ? $data['id'] : 0;
        $storeId = $this->storeManager->getStore()->getId();
        $this->setScope($storeId);

        $newImages = $data['images'];
        $upload = isset($data['upload']) ? $data['upload'] : true;
        /**
         * @var Product $product
         */
        $product = $this->productRepository->getById($productId, true, $storeId, true);

        try {
            $resolverContext = $this->entriesResolverContextFactory->create([
                'product' => $product,
                'processorContext' => $this->processorContextFactory->create([
                    'context' => [
                        EntriesResolverContextInterface::FORCE_CLEAN_GALLERY => $data[EntriesResolverContextInterface::FORCE_CLEAN_GALLERY] ?? false,
                        EntriesResolverContextInterface::PRESERVE_PREVIOUS_IMAGES => $data[EntriesResolverContextInterface::PRESERVE_PREVIOUS_IMAGES] ?? false
                    ]
                ]),
                'operationType' => $upload ? EntriesResolverContextInterface::OPERATION_CREATE : EntriesResolverContextInterface::OPERATION_UPDATE,
                'imageEntries' => $newImages
            ]);
            $this->productMediaGalleryProcessor->execute($product, $resolverContext);
        } catch (Exception $e) {
            throw new WebapiException(__($e->getMessage()), 403);
        }

        return $response;
    }

    /**
     * Sets scope to admin if necessary
     *
     * @param int $storeId
     *
     * @return $this
     */
    private function setScope($storeId)
    {
        if ($storeId === 0) {
            // Change scope code to admin
            $storeResolver = $this->objectManager->get(\Magento\Store\Api\StoreResolverInterface::class);
            $closure = function () {
                $this->scopeCode = 'admin';
            };
            $closure = \Closure::bind($closure, $storeResolver, $storeResolver);
            $closure();
        }

        $this->storeManager->setCurrentStore($storeId);

        return $this;
    }
}
