<?php

namespace SyncSpider\APIextension\Model\ProductMediaGallery;

use Magento\Framework\Filesystem;
use Magento\Framework\App\Filesystem\DirectoryList;
use Magento\Framework\Api\Data\{
    ImageContentInterface,
    ImageContentInterfaceFactory
};
use Magento\Framework\Filesystem\Directory\ReadInterface as DirectoryReadInterface;
use Magento\Catalog\Api\Data\ProductAttributeMediaGalleryEntryInterface;
use Magento\Catalog\Model\Product\Media\ConfigInterface as ProductMediaConfigInterface;
use SyncSpider\APIextension\Model\ProductMediaGallery\{
    EntriesResolver\CreateEntries,
    EntriesResolverContextInterface
};

class AreMediaGalleryEntriesEqual implements AreMediaGalleryEntriesEqualInteface
{
    /**
     * @var Filesystem
     */
    private $filesystem;

    /**
     * @var ImageContentInterfaceFactory
     */
    private $imageContentFactory;

    /**
     * @var ProductMediaConfigInterface
     */
    private $productMediaConfig;

    /**
     * @var CreateEntries
     */
    private $createEntriesResolver;

    /**
     * @var DirectoryReadInterface|null
     */
    private $mediaDirectory;

    public function __construct(
        Filesystem $filesystem,
        ImageContentInterfaceFactory $imageContentFactory,
        ProductMediaConfigInterface $productMediaConfig,
        CreateEntries $createEntriesResolver
    ) {
        $this->filesystem = $filesystem;
        $this->imageContentFactory = $imageContentFactory;
        $this->productMediaConfig = $productMediaConfig;
        $this->createEntriesResolver = $createEntriesResolver;
    }

    /**
     * Get media directory read
     *
     * @return DirectoryReadInterface
     */
    public function getMediaDirectory(): DirectoryReadInterface
    {
        if (! isset($this->mediaDirectory)) {
            $this->mediaDirectory = $this->filesystem->getDirectoryRead(DirectoryList::MEDIA);  
        }

        return $this->mediaDirectory;
    }

    /**
     * Get givn gallery image content
     *
     * @param ProductAttributeMediaGalleryEntryInterface $image
     *
     * @return ImageContentInterface|null
     */
    public function getGalleryImageContent(ProductAttributeMediaGalleryEntryInterface $image): ?ImageContentInterface
    {
        $content = $image->getContent();
        $file = $image->getFile();
        $fileMediaPath = filter_var($file, FILTER_VALIDATE_URL) ? null : $this->productMediaConfig->getMediaPath($file);

        if (! $content
            && $image->getId()
            && $this->getMediaDirectory()->isExist($fileMediaPath)
        ) {
            $content = $this->imageContentFactory->create();
            $content->setBase64EncodedData(base64_encode(
                $this->getMediaDirectory()->readFile($fileMediaPath)
            ));
            $image->setContent($content);
        }

        return $content;
    }

    /**
     * Get base gallery image name
     * 
     * Base gallery image name is name with the duplicate counter removed,
     * file extension removed and normalized string value. For example, an value
     * "TB_400_x_520_px_Umreifungsband_100.png" would be "TB-400-x-520-px-Umreifungsband"
     *
     * @param ProductAttributeMediaGalleryEntryInterface $galleryImage
     *
     * @return string
     */
    public function getBaseGalleryImageName(ProductAttributeMediaGalleryEntryInterface $galleryImage): string
    {
        $fileName = pathinfo($galleryImage->getFile(), PATHINFO_FILENAME);
        $fileNameParts = explode('_', $fileName);
        $counter = array_pop($fileNameParts);

        if (! is_numeric($counter)) {
            $fileNameParts[] = $counter;
        }

        return $this->createEntriesResolver->normalizeString(implode('_', $fileNameParts));
    }


    /**
     * Image equality implementation
     *
     * Images are equal in following cases:
     * - their base64 encoded data are equal
     * - their base gallery image names are equal in case
     *   at least one of the entries is new entry without image content (UPDATE operation) 
     *
     * {@inheritDoc}
     */
    public function execute(
        ProductAttributeMediaGalleryEntryInterface $entryOne,
        ProductAttributeMediaGalleryEntryInterface $entryTwo,
        EntriesResolverContextInterface $context = null
    ): bool {
        $contentOne = $this->getGalleryImageContent($entryOne);
        $contentTwo = $this->getGalleryImageContent($entryTwo);

        // compare base64 contents in case contents available
        if ($contentOne && $contentTwo) {
            return $contentOne->getBase64EncodedData() === $contentTwo->getBase64EncodedData();
        }

        // one of the entries is new entry without image content (UPDATE operation)
        if ((! $entryOne->getId() && ! $contentOne) || (! $entryTwo->getId() && ! $contentTwo)) {
            // base gallery image names are equal
            if ($this->getBaseGalleryImageName($entryOne) === $this->getBaseGalleryImageName($entryTwo)) {
                return true;
            }

            if ($context) {
                $syncspiderImageOne = $context->getSyncspiderImage($entryOne);
                $syncspiderImageTwo = $context->getSyncspiderImage($entryTwo);

                // syncspider image source url are equal
                if ($syncspiderImageOne && $syncspiderImageTwo) {
                    return $syncspiderImageOne->getSourceUrl() === $syncspiderImageTwo->getSourceUrl();
                }
            }
        }

        return false;
    }
}