<?php

namespace SyncSpider\APIextension\Model\ProductMediaGallery\EntriesResolver;

use Magento\Framework\Api\Data\{
    ImageContentInterface,
    ImageContentInterfaceFactory
};
use Magento\Catalog\Api\Data\{
    ProductAttributeMediaGalleryEntryInterface,
    ProductAttributeMediaGalleryEntryInterfaceFactory
};
use SyncSpider\APIextension\Model\ProductMediaGallery\{
    EntriesResolverContextInterface,
    EntriesResolverInterface
};
use SyncSpider\APIextension\Api\Data\{
    SyncspiderImageInterface,
    SyncspiderImageInterfaceFactory
};
use SyncSpider\APIextension\Model\ProductMediaGallery\EntriesResolver\CleanEntriesTypes;

class CreateEntries implements EntriesResolverInterface
{
    /**
     * @var ImageContentInterfaceFactory
     */
    private $imageContentFactory;

    /**
     * @var ProductAttributeMediaGalleryEntryInterfaceFactory
     */
    private $mediaGalleryEntryFactory;

    /**
     * @var SyncspiderImageInterfaceFactory
     */
    private $syncspiderImageFactory;

    /**
     * @var CleanEntriesTypes
     */
    private $cleanEntriesTypes;

    public function __construct(
        ImageContentInterfaceFactory $imageContentFactory,
        ProductAttributeMediaGalleryEntryInterfaceFactory $mediaGalleryEntryFactory,
        SyncspiderImageInterfaceFactory $syncspiderImageFactory,
        CleanEntriesTypes $cleanEntriesTypes
    ) {
        $this->imageContentFactory = $imageContentFactory;
        $this->mediaGalleryEntryFactory = $mediaGalleryEntryFactory;
        $this->syncspiderImageFactory = $syncspiderImageFactory;
        $this->cleanEntriesTypes = $cleanEntriesTypes;
    }

    /**
     * Normalize given string value in common format
     *
     * @param string $value
     *
     * @return string
     */
    public function normalizeString(string $value): string
    {
        return str_replace('_', '-', strtolower($value));
    }

    /**
     * Get image content for given image url
     *
     * Image content is created only in case of create operation type context
     *
     * @param string $imageUrl
     * @param EntriesResolverContextInterface $context
     *
     * @return ImageContentInterface|null
     */
    public function getImageContent(string $imageUrl, EntriesResolverContextInterface $context) : ?ImageContentInterface
    {
        $content = null;

        // download and populate content only if requested
        if ($context->isCreateOperation()) {
            $imageContent = file_get_contents($imageUrl);
            $fileInfo = getimagesizefromstring($imageContent);
            $content = $this->imageContentFactory->create()
                ->setBase64EncodedData(base64_encode($imageContent))
                ->setName($this->normalizeString(pathinfo($imageUrl, PATHINFO_BASENAME)))
                ->setType($fileInfo['mime']);
        }

        return $content;
    }

    /**
     * Create media gallery entry based on given data
     *
     * @param array $data
     * @param EntriesResolverContextInterface $context
     *
     * @return ProductAttributeMediaGalleryEntryInterface
     */
    public function getMediaGalleryEntry(array $data, EntriesResolverContextInterface $context): ProductAttributeMediaGalleryEntryInterface
    {
        // create media gallery entry
        $mediaGalleryEntry = $this->mediaGalleryEntryFactory->create()
            ->setDisabled(false)
            ->setLabel($data[ProductAttributeMediaGalleryEntryInterface::LABEL])
            ->setTypes($data[ProductAttributeMediaGalleryEntryInterface::TYPES])
            ->setMediaType('image')
            ->setContent($data[ProductAttributeMediaGalleryEntryInterface::CONTENT])
            ->setFile($context->isCreateOperation() ? null : $data[SyncspiderImageInterface::SOURCE_URL]);// needs to be removed before saving
        // create syncspider image extension attribute
        $syncspiderImage = $this->syncspiderImageFactory->create([
            'data' => [
                SyncspiderImageInterface::SOURCE_URL => $data[SyncspiderImageInterface::SOURCE_URL],
                SyncspiderImageInterface::IS_FRESH => true
            ]
        ]);
        // attach syncspider image extension attribute to media gallery entry
        $extensionAttributes = $context->getProcessorContext()->getEntityExtensionAttributes($mediaGalleryEntry);
        $extensionAttributes->setSyncspiderImage($syncspiderImage);
        return $mediaGalleryEntry;
    }

    /**
     * Attach new media gallery entries based on context image entries
     *
     * New media entries are added at the end of the given emdia gallery
     * entries
     *
     * {@inheritDoc}
     */
    public function getMediaGalleryEntries(array $mediaGalleryEntries, EntriesResolverContextInterface $context): array
    {
        $imageEntries = $context->getImageEntries();
        $newEntries = [];
        $contents = [];

        foreach ($imageEntries as $imageEntry) {
            $imageUrl = $imageEntry['url'] ?? '';
            $types = $imageEntry['types'] ?? [];
            $label = $imageEntry['label'] ?? '';

            // skip image entries without valid url
            if (! filter_var($imageUrl, FILTER_VALIDATE_URL)) {
                continue;
            }

            if (! array_key_exists($imageUrl, $contents)) {
                $contents[$imageUrl] = $this->getImageContent($imageUrl, $context);
            }

            $content = $contents[$imageUrl];
            $newEntries[] = $this->getMediaGalleryEntry([
                SyncspiderImageInterface::SOURCE_URL => $imageUrl,
                ProductAttributeMediaGalleryEntryInterface::LABEL => $label,
                ProductAttributeMediaGalleryEntryInterface::TYPES => $types,
                ProductAttributeMediaGalleryEntryInterface::CONTENT => $content
            ], $context);
        }

        return array_merge(
            $mediaGalleryEntries,
            $newEntries
        );
    }
}