<?php

namespace SyncSpider\APIextension\Model\ProductMediaGallery\EntriesResolver;

use Magento\Framework\App\ResourceConnection;
use Magento\Catalog\Api\Data\{
    ProductAttributeMediaGalleryEntryInterface,
    ProductInterface
};
use SyncSpider\APIextension\Api\EntityTypeRepositoryInterface;
use SyncSpider\APIextension\EntityManager\SyncspiderSynchronizationDataHandler;
use SyncSpider\APIextension\Model\ProductMediaGallery\{
    EntriesResolverContextInterface,
    EntriesResolverInterface
};

class ProcessRemovedEntries implements EntriesResolverInterface
{
    /**
     * @var ResourceConnection
     */
    private $resourceConnection;

    /**
     * @var EntityTypeRepositoryInterface
     */
    private $entityTypeRepository;

    /**
     * @var SyncspiderSynchronizationDataHandler
     */
    private $selectProcessor;

    public function __construct(
        ResourceConnection $resourceConnection,
        EntityTypeRepositoryInterface $entityTypeRepository,
        SyncspiderSynchronizationDataHandler $selectProcessor
    ) {
        $this->resourceConnection = $resourceConnection;
        $this->entityTypeRepository = $entityTypeRepository;
        $this->selectProcessor = $selectProcessor;
    }

    /**
     * Process removed media gallery entries
     * 
     * Cleans references to media gallery entries so the media gallery images
     * can be correctly removed. Cleans the "catalog_product_entity_varchar" table
     * referencing media gallery entry files which are used as values for
     * "image"/"small_image"/"thumbnail" attributes
     *
     * {@inheritDoc}
     */
    public function getMediaGalleryEntries(array $mediaGalleryEntries, EntriesResolverContextInterface $context): array
    {
        $product = $context->getProduct();
        $existingEntriesMap = $context->getMediaGalleryEntriesMap($product->getMediaGalleryEntries());
        $updatedEntriesMap = $context->getMediaGalleryEntriesMap($mediaGalleryEntries);
        $valueBindNames = array_reduce(
            array_diff_key($existingEntriesMap, $updatedEntriesMap),
            function (array $carry, ProductAttributeMediaGalleryEntryInterface $entry) {
                if ($entry->getFile()) {
                    $carry[] = $entry->getFile();
                }

                return $carry;
            },
            []
        );

        if (! empty($valueBindNames)) {
            $connection = $this->resourceConnection->getConnection();
            $select = $connection->select();
            $entityTypeId = $this->entityTypeRepository->getEntityTypeId(ProductInterface::class);
            $valueBindNames = $this->selectProcessor->bindValues($select, $valueBindNames, 'media_value');
            $attributeSelect = $connection->select()
                ->from(
                    ['eav' => $this->resourceConnection->getTableName('eav_attribute')],
                    'eav.attribute_id'
                )
                ->where('eav.entity_type_id = :entity_type_id')
                ->where('eav.backend_type = :eav_varchar_backend');
            $select->from(
                    ['cpev' => $this->resourceConnection->getTableName('catalog_product_entity_varchar')],
                    'cpev.entity_id'
                )
                ->where('cpev.entity_id = :entity_id')
                ->where('cpev.attribute_id IN (' . $attributeSelect->assemble() .')')
                ->where('cpev.value IN(' . implode(',', $valueBindNames) . ')');
            $deleteQuery = $connection->deleteFromSelect($select, 'cpev');
            $connection->query(
                $deleteQuery,
                array_merge($select->getBind(), [
                    'entity_id' => $product->getId(),
                    'entity_type_id' => $entityTypeId,
                    'eav_varchar_backend' => 'varchar'
                ])
            );
        }

        return $mediaGalleryEntries;
    }
}