<?php

namespace SyncSpider\APIextension\Model\ProductMediaGallery\EntriesResolver;

use Magento\Catalog\Api\{
    Data\ProductAttributeMediaGalleryEntryInterface,
    ProductRepositoryInterface
};
use SyncSpider\APIextension\Model\ResourceModel\Image\CollectionFactory as ImageCollectionFactory;
use SyncSpider\APIextension\Model\ProductMediaGallery\{
    EntriesResolverContextInterface,
    EntriesResolverInterface
};

class SaveEntries implements EntriesResolverInterface
{
    /**
     * @var ProductRepositoryInterface
     */
    private $productRepository;

    /**
     * @var ImageCollectionFactory
     */
    private $imageCollectionFactory;

    public function __construct(
        ProductRepositoryInterface $productRepository,
        ImageCollectionFactory $imageCollectionFactory,
    ) {
        $this->productRepository = $productRepository;
        $this->imageCollectionFactory = $imageCollectionFactory;
    }

    /**
     * Save given media gallery entries for context product
     *
     * Save relation of media gallery entry with iths source
     * syncspider image data
     *
     * {@inheritDoc}
     */
    public function getMediaGalleryEntries(array $mediaGalleryEntries, EntriesResolverContextInterface $context): array
    {
        $mapKeyFn = function (ProductAttributeMediaGalleryEntryInterface $entry) {
            return $entry->getPosition();
        };
        $entriesMapSource = $context->getMediaGalleryEntriesMap($mediaGalleryEntries, $mapKeyFn);
        $mediaGalleryEntries = $this->saveMediaGalleryEntries($mediaGalleryEntries, $context);
        $entriesMap = $context->getMediaGalleryEntriesMap($mediaGalleryEntries, $mapKeyFn);
        $rows = $this->generateSyncspiderImageRows($entriesMap, $entriesMapSource, $context);
        $this->saveSyncspiderImageRows($rows);
        return $mediaGalleryEntries;
    }

    /**
     * Save given media entries for given context product
     *
     * @param array $mediaGalleryEntries
     * @param EntriesResolverContextInterface $context
     *
     * @return ProductAttributeMediaGalleryEntryInterface[]
     */
    public function saveMediaGalleryEntries(array $mediaGalleryEntries, EntriesResolverContextInterface $context): array
    {
        $product = $context->getProduct();
        $product->setMediaGalleryEntries($mediaGalleryEntries);
        $product = $this->productRepository->save($product);
        return $product->getMediaGalleryEntries();
    }

    /**
     * Save given syncspider image rows
     *
     * @param array $rows
     *
     * @return bool
     */
    public function saveSyncspiderImageRows(array $rows): bool
    {
        if (! empty($rows)) {
            $metadata = $this->imageCollectionFactory->create();
            $connection = $metadata->getConnection();
            $connection->insertOnDuplicate(
                $metadata->getMainTable(),
                $rows,
                ['tag']
            );
        }

        return false;
    }

    /**
     * Generate syncspider image rows
     *
     * @param array $entriesMap Map with saved/fresh entries
     * @param array $entriesMapSource  Map with entries having source syncspider image data  
     * @param EntriesResolverContextInterface $context
     *
     * @return array
     */
    public function generateSyncspiderImageRows(
        array $entriesMap,
        array $entriesMapSource,
        EntriesResolverContextInterface $context
    ): array {
        $rows = [];

        foreach ($entriesMap as $position => $entry) {
            $sourceEntry = $entriesMapSource[$position] ?? null;
            $syncspiderImage = $sourceEntry ? $context->getSyncspiderImage($sourceEntry) : null;

            if ($syncspiderImage) {
                $rows[] = [
                    'image_id' => $entry->getId(),
                    'tag' => $syncspiderImage->getSourceUrl()
                ];
            }
        }

        return $rows;
    }
}