<?php

namespace SyncSpider\APIextension\Model\ProductMediaGallery;

use Magento\Catalog\Api\Data\{
    ProductInterface,
    ProductAttributeMediaGalleryEntryInterface,
    ProductAttributeMediaGalleryEntryExtensionInterface
};
use SyncSpider\APIextension\Api\Data\SyncspiderImageInterface;
use SyncSpider\APIextension\Api\ExtensionAttributes\ProcessorContextInterface;

class EntriesResolverContext implements EntriesResolverContextInterface
{
    /**
     * @var ProductInterface
     */
    private $product;

    /**
     * @var ProcessorContextInterface
     */
    private $processorContext;

    /**
     * @var string
     */
    private $operationType;

    /**
     * @var array
     */
    private $imageEntries;

    public function __construct(
        ProductInterface $product,
        ProcessorContextInterface $processorContext,
        string $operationType,
        array $imageEntries = []
    ) {
        $this->product = $product;
        $this->processorContext = $processorContext;
        $this->operationType = $operationType;
        $this->imageEntries = $imageEntries;
    }

    /**
     * @inheritDoc
     */
    public function getProduct(): ProductInterface
    {
        return $this->product;
    }

    /**
     * @inheritDoc
     */
    public function getOperationType(): string
    {
        return $this->operationType;
    }

    /**
     * @inheritDoc
     */
    public function isCreateOperation(): bool
    {
        return $this->getOperationType() === static::OPERATION_CREATE;
    }

    /**
     * @inheritDoc
     */
    public function isUpdateOperation(): bool
    {
        return $this->getOperationType() === static::OPERATION_UPDATE;
    }

    /**
     * @inheritDoc
     */
    public function getImageEntries(): array
    {
        return $this->imageEntries;
    }

    /**
     * @inheritDoc
     */
    public function getProcessorContext(): ProcessorContextInterface
    {
        return $this->processorContext;
    }

    /**
     * @inheritDoc
     */
    public function isForceCleanGallery(): bool
    {
        return $this->isCreateOperation()
            && $this->getProcessorContext()->getContextValue(static::FORCE_CLEAN_GALLERY) === true;
    }

    /**
     * @inheritDoc
     */
    public function isPreservePreviousImages(): bool
    {
        return ! $this->isCreateOperation()
            || $this->getProcessorContext()->getContextValue(static::PRESERVE_PREVIOUS_IMAGES) === true;
    }

    /**
     * @inheritDoc
     */
    public function getSyncspiderImage(ProductAttributeMediaGalleryEntryInterface $entry): ?SyncspiderImageInterface
    {
        $processorContext = $this->getProcessorContext();
        
        if ($processorContext->validEntity($entry)) {
            $extensionAttributes = $processorContext->getEntityExtensionAttributes($entry);

            if ($extensionAttributes instanceof ProductAttributeMediaGalleryEntryExtensionInterface
                && $extensionAttributes->getSyncspiderImage() instanceof SyncspiderImageInterface
            ) {
                return $extensionAttributes->getSyncspiderImage();
            }
        }

        return null;
    }

    /**
     * @inheritDoc
     */
    public function getMediaGalleryEntriesMap(array $mediaGalleryEntries, callable $mapKeyFn = null): array
    {
        $defaultMapKeyFn = function (ProductAttributeMediaGalleryEntryInterface $entry) {
            return $entry->getId() ?: null;
        };
        $mapKeyFn = $mapKeyFn ?? $defaultMapKeyFn;
        return array_reduce(
            $mediaGalleryEntries,
            function (array $map, ProductAttributeMediaGalleryEntryInterface $entry) use ($mapKeyFn) {
                $mapKey = $mapKeyFn($entry);

                if ($mapKey) {
                    $map[$mapKey] = $entry;
                }

                return $map;
            },
            []
        );
    }
}
