<?php

namespace SyncSpider\APIextension\Model\ProductOption;

use Magento\Framework\{
    DataObject,
    DataObjectFactory
};
use Magento\Catalog\Api\Data\ProductOptionInterface;
use Magento\Catalog\Model\ProductOptionProcessorInterface;

class ProductOptionProcessor implements ProductOptionProcessorInterface, VisibleProductOptionProcessorInterface
{
    /**
     * @var DataObjectFactory
     */
    private $dataObjectFactory;

    /**
     * @var ProductOptionProcessorInterface[]
     */
    private $processorPool;

    public function __construct(DataObjectFactory $dataObjectFactory, array $processorPool = [])
    {
        $this->dataObjectFactory = $dataObjectFactory;
        $this->processorPool = $processorPool;
    }

    /**
     * @inheritDoc
     */
    public function convertToBuyRequest(ProductOptionInterface $productOption)
    {
        $buyRequest = [];

        foreach ($this->processorPool as $processor) {
            $request = $processor->convertToBuyRequest($productOption);
            $buyRequest = array_merge($buyRequest, $request->getData());
        }

        return $this->dataObjectFactory->create(['data' => $buyRequest]);
    }

    /**
     * @inheritDoc
     */
    public function convertToProductOption(DataObject $request)
    {
        $productOption = [];

        foreach ($this->processorPool as $processor) {
            $data = $processor->convertToProductOption($request);
            $productOption = array_merge($productOption, $data);
        }

        return $productOption;
    }

    /**
     * @inheritDoc
     */
    public function convertToVisibleOptions(ProductOptionInterface $productOption): array
    {
        $productOptions = [];

        foreach ($this->processorPool as $processor) {
            if ($processor instanceof VisibleProductOptionProcessorInterface) {
                $productOptions = array_reduce(
                    $processor->convertToVisibleOptions($productOption),
                    function (array $productOptions, array $option) {
                        $productOptions[] = [
                            'label' => $option['label'],
                            'value' => $option['value'],
                            'print_value' => $option['value'],
                            'option_type' => 'field',
                            'option_view' => false,
                            VisibleProductOptionProcessorInterface::READONLY => true
                        ];
                        return $productOptions;
                    },
                    $productOptions
                );
            }
        }

        return $productOptions;
    }

    /**
     * Remove read only options from given options
     *
     * @param array $options
     *
     * @return array
     */
    public function removeReadOnlyOptions(array $options): array
    {
        $options = array_filter(
            $options,
            function ($value) {
                $readOnly = $value[VisibleProductOptionProcessorInterface::READONLY] ?? false;
                return ! $readOnly;
            }
        );
        return $options;
    }
}
