<?php

namespace SyncSpider\APIextension\Model\ProductOption;

use Magento\Catalog\Api\Data\ProductOptionInterface;
use Magento\Catalog\Model\ProductOptionProcessorInterface;
use Magento\Framework\DataObject;
use Magento\Framework\DataObject\Factory as DataObjectFactory;
use SyncSpider\APIextension\{
    Api\Data\ProductOptionInterface as SyncSpiderProductOptionInterface,
    Api\Data\ProductOptionInterfaceFactory
};
use SyncSpider\APIextension\Model\ProductOption\VisibleProductOptionProcessorInterface;

class SimpleProductOptionProcessor implements ProductOptionProcessorInterface, VisibleProductOptionProcessorInterface
{
    /**
     * @var DataObjectFactory
     */
    private $dataObjectFactory;

    /**
     * @var ProductOptionInterfaceFactory
     */
    private $productOptionFactory;

    /**
     * ProductOptionProcessor constructor
     *
     * @param DataObjectFactory $dataObjectFactory
     * @param ProductOptionInterfaceFactory $productOptionFactory
     */
    public function __construct(
        DataObjectFactory $dataObjectFactory,
        ProductOptionInterfaceFactory $productOptionFactory
    ) {
        $this->dataObjectFactory = $dataObjectFactory;
        $this->productOptionFactory = $productOptionFactory;
    }

    /**
     * Get all available attributes data
     *
     * @param DataObject $request
     *
     * @return array
     */
    public function getAttributesInformation(DataObject $request)
    {
        $result = [];
        $paths = [
            'product_options/attributes_info',
            'product_options/options',
            SyncSpiderProductOptionInterface::ATTRIBUTE_CODE
        ];

        foreach ($paths as $path) {
            $data = $request->getData($path);

            if (is_array($data)) {
                foreach ($data as $attributeData) {
                    if (is_array($attributeData)) {
                        $readOnly = $attributeData[VisibleProductOptionProcessorInterface::READONLY] ?? false;
                        $attributeData = array_intersect_key($attributeData, ['label' => true, 'value' => true]);
                        $attributeData = array_filter($attributeData, function ($value) {
                            return is_scalar($value);
                        });

                        if (! $readOnly && count($attributeData) === 2) {
                            $result[] = [
                                'label' => $attributeData['label'],
                                'value' => $attributeData['value'],
                                'path'  => $path
                            ];
                        }
                    }
                }
            }
        }

        return $result;
    }

    /**
     * @inheritdoc
     */
    public function convertToProductOption(DataObject $request)
    {
        $attributesInfo = $this->getAttributesInformation($request);
        $data = [];

        if (is_array($attributesInfo)) {
            foreach ($attributesInfo as $option) {
                $productOption = $this->productOptionFactory->create();
                $productOption->setLabel($option['label'])
                    ->setValue($option['value'])
                    ->setPath($option['path']);
                $data[] = $productOption;
            }
        }

        if (! empty($data)) {
            $data = [
                SyncSpiderProductOptionInterface::ATTRIBUTE_CODE => $data
            ];
        }

        return $data;
    }

    /**
     * @inheritdoc
     */
    public function convertToBuyRequest(ProductOptionInterface $productOption)
    {
        $request = $this->dataObjectFactory->create();
        $attributesInfo = [];

        if ($productOption->getExtensionAttributes()
            && $productOption->getExtensionAttributes()->getSimpleProductOptions()
        ) {
            $simpleOptions = $productOption->getExtensionAttributes()->getSimpleProductOptions();
            $attributesInfo = array_reduce(
                is_array($simpleOptions) ? $simpleOptions : [],
                function (array $attributesInfo, SyncSpiderProductOptionInterface $productOption) {
                    if ($productOption->getPath() === SyncSpiderProductOptionInterface::ATTRIBUTE_CODE) {
                        $attributesInfo[] = [
                            'label' => $productOption->getLabel(),
                            'value' => $productOption->getValue(),
                            'path' => $productOption->getPath()
                        ];
                    }

                    return $attributesInfo;
                },
                []
            );
        }

        if ($attributesInfo) {
            $request->setData(SyncSpiderProductOptionInterface::ATTRIBUTE_CODE, $attributesInfo);
        }

        return $request;
    }

    /**
     * @inheritDoc
     */
    public function convertToVisibleOptions(ProductOptionInterface $productOption): array
    {
        $request = $this->convertToBuyRequest($productOption);
        $options = $request->getData(SyncSpiderProductOptionInterface::ATTRIBUTE_CODE);
        return is_array($options) ? $options : [];
    }
}
