<?php

namespace SyncSpider\APIextension\Model;

use Magento\Framework\Filesystem;
use Magento\Framework\App\Filesystem\DirectoryList;
use Magento\Framework\Encryption\EncryptorInterface;
use Magento\Framework\Filesystem\Directory\WriteInterface;
use Magento\Framework\Image\AdapterFactory;
use Magento\MediaStorage\Model\File\UploaderFactory as MediaStorageUploaderFactory;
use Magento\Catalog\Model\Product\Media\Config as CatalogMediaConfig;
use Magento\Swatches\Helper\Media as SwatchMedia;
use SyncSpider\APIextension\Api\{
    RemoteResourceInterface,
    RemoteResourceInterfaceFactory,
    UploaderInterface
};
use Exception;

class Uploader implements UploaderInterface
{
    /**
     * @var Filesystem
     */
    private $filesystem;

    /**
     * @var EncryptorInterface
     */
    private $encryptor;

    /**
     * @var AdapterFactory
     */
    private $imageAdapterFactory;

    /**
     * @var MediaStorageUploaderFactory
     */
    private $mediaStorageUploaderFactory;

    /**
     * @var CatalogMediaConfig
     */
    private $catalogMediaConfig;

    /**
     * @var SwatchMedia
     */
    private $swatchMedia;

    /**
     * @var RemoteResourceInterfaceFactory
     */
    private $remoteResourceFactory;

    public function __construct(
        Filesystem $filesystem,
        EncryptorInterface $encryptor,
        AdapterFactory $imageAdapterFactory,
        MediaStorageUploaderFactory $mediaStorageUploaderFactory,
        CatalogMediaConfig $catalogMediaConfig,
        SwatchMedia $swatchMedia,
        RemoteResourceInterfaceFactory $remoteResourceFactory
    ) {
        $this->filesystem = $filesystem;
        $this->encryptor = $encryptor;
        $this->imageAdapterFactory = $imageAdapterFactory;
        $this->mediaStorageUploaderFactory = $mediaStorageUploaderFactory;
        $this->catalogMediaConfig = $catalogMediaConfig;
        $this->swatchMedia = $swatchMedia;
        $this->remoteResourceFactory = $remoteResourceFactory;
    }

    /**
     * Get temporary directory
     *
     * @return WriteInterface
     */
    public function getTmpDirectory(): WriteInterface
    {
        return $this->filesystem->getDirectoryWrite(DirectoryList::SYS_TMP);
    }

    /**
     * Get media directory
     *
     * @return WriteInterface
     */
    public function getMediaDirectory(): WriteInterface
    {
        return $this->filesystem->getDirectoryWrite(DirectoryList::MEDIA);
    }

    /**
     * Get resource for remote source url
     *
     * @param string $sourceUrl
     *
     * @return RemoteResourceInterface|null
     */
    public function getRemoteResource(string $sourceUrl): ?RemoteResourceInterface
    {
        $resource = false;

        if (filter_var($sourceUrl, FILTER_VALIDATE_URL)) {
            try {
                $resource = fopen($sourceUrl, 'r', false);
                $resource = $this->remoteResourceFactory->create([
                    'resource' => $resource
                ]);
            } catch (Exception $e) {
                // suppress exceptions
            }
        }
        
        return $resource ?? null;
    }

    /**
     * Get list of allowed mime types for given type
     *
     * @param string $type
     *
     * @return array
     */
    public function getAllowedMimeTypes(string $type): array
    {
        switch ($type) {
            case 'image':
                return [
                    'image/jpeg',
                    'image/gif',
                    'image/png'
                ];
        }

        return [];
    }

    /**
     * Whether given remote resource mime type is valid
     *
     * @param RemoteResourceInterface $resource
     * @param array $allowedMimeTypes
     *
     * @return boolean
     */
    public function isResourceMimeValid(RemoteResourceInterface $remoteResource, array $allowedMimeTypes): bool
    {
        $mime = $remoteResource->getMimeType();
        return in_array($mime, $allowedMimeTypes);
    }

    /**
     * Save remote image from given source url
     *
     * @param string $sourceUrl
     *
     * @return array|null
     */
    public function saveRemoteImage(string $sourceUrl): ?array
    {
        $remoteResource = $this->getRemoteResource($sourceUrl);

        try {
            if ($remoteResource
                && $this->isResourceMimeValid($remoteResource, $this->getAllowedMimeTypes('image'))
            ) {
                $imageContent =  $remoteResource->getContents();
                $fileInfo = getimagesizefromstring($imageContent);
                $name = pathinfo($sourceUrl,  PATHINFO_BASENAME);
                $sourceUrlHash = $this->encryptor->hash($sourceUrl);
                $tmpName = $sourceUrlHash . '.' . $name;
                $tmpDirectory = $this->getTmpDirectory();
                $tmpDirectory->writeFile($tmpName, $imageContent);
                $fileAttributes = [
                    'tmp_name' => $tmpDirectory->getAbsolutePath() . $tmpName,
                    'name' => $name
                ];
                return [
                    $tmpName,
                    $name,
                    $fileInfo,
                    $fileAttributes
                ];
            }
        } catch (Exception $e) {
            // supress exceptions on image saving as temporary file
        }

        return null;
    }

    /**
     * @inheritDoc
     */
    public function saveSwatchImage(string $sourceUrl): ?string
    {
        $data = $this->saveRemoteImage($sourceUrl);

        if ($data) {
            try {
                [$tmpName, $name, $fileInfo, $fileAttributes] = $data;
                $imageAdapter = $this->imageAdapterFactory->create();
                $uploader = $this->mediaStorageUploaderFactory->create(['fileId' => $fileAttributes]);
                $uploader->addValidateCallback('catalog_product_image', $imageAdapter, 'validateUploadFile')
                    ->setAllowedExtensions(['jpg', 'jpeg', 'gif', 'png'])
                    ->setAllowRenameFiles(true)
                    ->setFilesDispersion(true);
                $result = $uploader->save(
                    $this->getMediaDirectory()->getAbsolutePath(
                        $this->catalogMediaConfig->getBaseTmpMediaPath()
                    )
                );
                $file = $result['file'] ?? null;

                if ($file) {
                    $file = $this->swatchMedia->moveImageFromTmp($file);
                    $this->swatchMedia->generateSwatchVariations($file);
                    return $file;
                }
            } catch (Exception $e) {
                // suppress exceptions on saving temporary file as swatch image
            }
        }

        return null;
    }
}
