<?php
namespace SyncSpider\APIextension\Model;

use Magento\Framework\Api\SearchCriteriaInterface;
use Magento\Framework\Api\SortOrder;
use Magento\Framework\App\ObjectManager;
use Magento\Framework\Exception\NoSuchEntityException;
use SyncSpider\APIextension\Api\Data\WebhookInterface;
use SyncSpider\APIextension\Api\Data\WebhookSearchResultInterfaceFactory;
use SyncSpider\APIextension\Api\WebhookRepositoryInterface;
use SyncSpider\APIextension\Helper\Data as SyncSpiderHelper;
use SyncSpider\APIextension\Model\WebhookFactory;
use SyncSpider\APIextension\Model\ResourceModel\Webhook as WebhookResourceModel;
use SyncSpider\APIextension\Model\ResourceModel\Webhook\Collection as WebhookCollection;
use SyncSpider\APIextension\Model\ResourceModel\Webhook\CollectionFactory as WebhookCollectionFactory;

class WebhookRepository implements WebhookRepositoryInterface
{
    /**
     * @var \SyncSpider\APIextension\Model\ResourceModel\Webhook
     */
    protected $webhookResource;
    /**
     * @var WebhookFactory
     */
    private $webhookFactory;

    /**
     * @var WebhookCollectionFactory
     */
    private $webhookCollectionFactory;

    /**
     * @var WebhookSearchResultInterfaceFactory
     */
    private $searchResultFactory;

    /**
     * @var SyncSpiderHelper
     */
    private $helper;

    /**
     * WebhookRepository constructor.
     *
     * @param WebhookFactory $webhookFactory
     * @param WebhookCollectionFactory $webhookCollectionFactory
     * @param WebhookSearchResultInterfaceFactory $webhookSearchResultInterfaceFactory
     * @param WebhookResourceModel $webhookResource
     * @param SyncSpiderHelper $helper
     */
    public function __construct(
        WebhookFactory $webhookFactory,
        WebhookCollectionFactory $webhookCollectionFactory,
        WebhookSearchResultInterfaceFactory $webhookSearchResultInterfaceFactory,
        WebhookResourceModel $webhookResource,
        SyncSpiderHelper $helper
    ) {
        $this->webhookFactory = $webhookFactory;
        $this->webhookCollectionFactory = $webhookCollectionFactory;
        $this->searchResultFactory = $webhookSearchResultInterfaceFactory;
        $this->webhookResource = $webhookResource;
        $this->helper = $helper;
    }

    /**
     * @inheritDoc
     */
    public function getList(SearchCriteriaInterface $searchCriteria)
    {
        $collection = $this->webhookCollectionFactory->create();
        $this->addFiltersToCollection($searchCriteria, $collection);
        $this->addSortOrdersToCollection($searchCriteria, $collection);
        $this->addPagingToCollection($searchCriteria, $collection);
        $collection->load();
        return $this->buildSearchResult($searchCriteria, $collection);
    }

    /**
     * @inheritDoc
     */
    public function getById($id)
    {
        $webhook = $this->getModelById($id);
        return [
            $webhook->getData()
        ];
    }

    /**
     * Get webhook model by id
     *
     * @param int $id
     *
     * @return array
     *
     * @throws NoSuchEntityException
     */
    public function getModelById($id)
    {
        $webhook = $this->webhookFactory->create();
        $this->webhookResource->load($webhook, $id);

        if (! $webhook->getId()) {
            throw new NoSuchEntityException(__('Unable to find webhook with ID "%1"', $id));
        }

        return $webhook;
    }

    /**
     * @inheritDoc
     */
    public function save(WebhookInterface $webhook)
    {
        $model = $this->webhookFactory->create();
        $model->setData([
            'event_code' => $webhook->getEventCode(),
            'active' => $webhook->getActive(),
            'callback_url' => $webhook->getCallbackUrl(),
            'description' => $webhook->getDescription(),
            'token' => $webhook->getToken(),
        ]);

        try {
            $this->webhookResource->save($model);
            return $this->getById($model->getId());
        } catch (\Exception $e) {
            return $e->getMessage();
        }
    }

    /**
     * @inheritDoc
     */
    public function delete($id)
    {
        $this->helper->log("Deletion of webhook with id: $id");
        $this->webhookResource->delete($this->getModelById($id));
    }

    /**
     * Add filters from given search criteria to given collecction
     *
     * @param SearchCriteriaInterface  $searchCriteria
     * @param WebhookCollectionFactory $collection
     *
     * @return void
     */
    private function addFiltersToCollection(SearchCriteriaInterface $searchCriteria, WebhookCollection $collection)
    {
        foreach ($searchCriteria->getFilterGroups() as $filterGroup) {
            $fields = $conditions = [];

            foreach ($filterGroup->getFilters() as $filter) {
                $fields[] = $filter->getField();
                $conditions[] = [$filter->getConditionType() => $filter->getValue()];
            }

            $collection->addFieldToFilter($fields, $conditions);
        }
    }

    /**
     * Add sort orders from given search criteria to given collecction
     *
     * @param SearchCriteriaInterface $searchCriteria
     * @param WebhookCollection $collection
     *
     * @return void
     */
    private function addSortOrdersToCollection(SearchCriteriaInterface $searchCriteria, WebhookCollection $collection)
    {
        foreach ((array)$searchCriteria->getSortOrders() as $sortOrder) {
            $direction = $sortOrder->getDirection() == SortOrder::SORT_ASC ? 'asc' : 'desc';
            $collection->addOrder($sortOrder->getField(), $direction);
        }
    }

    /**
     * Add pagination from given search criteria to given collecction
     *
     * @param SearchCriteriaInterface $searchCriteria
     * @param WebhookCollection $collection
     *
     * @return void
     */
    private function addPagingToCollection(SearchCriteriaInterface $searchCriteria, WebhookCollection $collection)
    {
        $collection->setPageSize($searchCriteria->getPageSize());
        $collection->setCurPage($searchCriteria->getCurrentPage());
    }

    /**
     * Build webhook search result
     *
     * @param SearchCriteriaInterface $searchCriteria
     * @param WebhookCollection $collection
     *
     * @return mixed
     */
    private function buildSearchResult(SearchCriteriaInterface $searchCriteria, WebhookCollection $collection)
    {
        $searchResults = $this->searchResultFactory->create();
        $searchResults->setSearchCriteria($searchCriteria);
        $searchResults->setItems($collection->getItems());
        $searchResults->setTotalCount($collection->getSize());
        return $searchResults;
    }
}
