<?php

namespace SyncSpider\APIextension\Plugin\Magento\Catalog\Api;

use Magento\Framework\Webapi\ServiceOutputProcessor;
use Magento\Framework\Webapi\ServiceInputProcessor;
use Magento\Catalog\Api\Data\ProductInterface;
use Magento\Catalog\Api\ProductRepositoryInterface;
use SyncSpider\APIextension\Api\WebapiServiceInputInterface;
use Exception;

class ProductRepositoryInterfacePlugin
{
    /**
     * @var WebapiServiceInputInterface
     */
    private $webapiServiceInput;

    /**
     * @var ServiceInputProcessor
     */
    private $serviceInputProcessor;

    /**
     * @var ServiceOutputProcessor
     */
    private $serviceOutputProcessor;

    /**
     * ProductRepositoryPlugin constructor.
     *
     * @param WebapiServiceInputInterface $webapiServiceInput
     * @param ServiceInputProcessor $serviceInputProcessor
     * @param ServiceOutputProcessor $serviceOutputProcessor
     */
    public function __construct(
        WebapiServiceInputInterface $webapiServiceInput,
        ServiceInputProcessor $serviceInputProcessor,
        ServiceOutputProcessor $serviceOutputProcessor
    ) {
        $this->webapiServiceInput = $webapiServiceInput;
        $this->serviceInputProcessor = $serviceInputProcessor;
        $this->serviceOutputProcessor = $serviceOutputProcessor;
    }

    /**
     * Get list of keys to remove from existing product data to prevent errors in saving when inheriting the data
     *
     * @return array
     */
    protected function getKeysToRemove()
    {
        return [
            'extension_attributes',
            'media_gallery',
            'media_gallery_entries',
            'product_links',
            'created_at',
            'updated_at'
        ];
    }

    /**
     * Correctly allow partial update of product data by merging new product data in existing data
     *
     * @param ProductRepositoryInterface $subject
     * @param ProductInterface $product
     * @param bool $saveOptions
     *
     * @return array|null
     */
    public function beforeSave(ProductRepositoryInterface $subject, ProductInterface $product, $saveOptions = false)
    {
        $keysToRemove = $this->getKeysToRemove();

        if ($product->getId() || $product->getSku()) {
            try {
                if ($this->webapiServiceInput->has($product)) {
                    $existingProduct = $product->getId()
                        ? $subject->getById($product->getId())
                        : $subject->get($product->getSku());
                    $existingData = $this->serviceOutputProcessor->process(
                        $existingProduct,
                        ProductRepositoryInterface::class,
                        'save'
                    );
                    $newData = $this->webapiServiceInput->get($product);

                    // remove data from existing product which are not required for "inheritance" and breaks saving
                    foreach ($keysToRemove as $key) {
                        unset($existingData[$key]);
                    }

                    $data = array_replace($existingData, $newData);
                    return $this->serviceInputProcessor->process(
                        ProductRepositoryInterface::class,
                        'save',
                        [
                            'product' => $data,
                            'saveOptions' => $saveOptions
                        ]
                    );
                }

            } catch (Exception $e) {
                // suppress exceptions
            }

            return null;
        }
    }
}
