<?php

namespace SyncSpider\APIextension\Plugin\Magento\Framework\Api\SearchCriteria;

use Magento\Framework\Api\SearchCriteriaInterface;
use Magento\Framework\Api\SearchCriteria\CollectionProcessor;
use Magento\Framework\Api\SearchCriteriaBuilderFactory;
use Magento\Framework\Data\Collection\AbstractDb;
use Magento\Sales\Model\ResourceModel\Order\Collection;
use SyncSpider\APIextension\Api\FilterGroupProcessorInterface;

class CollectionProcessorPlugin
{
    /**
     * @var SearchCriteriaBuilderFactory
     */
    private $searchCriteriaBuilderFactory;

    /**
     * @var FilterGroupProcessorInterface[]
     */
    private $filterGroupProcessors;

    /**
     * CollectionProcessorPlugin constructor.
     *
     * @param SearchCriteriaBuilderFactory $searchCriteriaBuilderFactory
     * @param FilterGroupProcessorInterface[] $filterGroupProcessors
     */
    public function __construct(
        SearchCriteriaBuilderFactory $searchCriteriaBuilderFactory,
        array $filterGroupProcessors = []
    ) {
        $this->searchCriteriaBuilderFactory = $searchCriteriaBuilderFactory;
        $this->filterGroupProcessors = $filterGroupProcessors;
    }

    /**
     * Apply custom filter group processors for collection based on the search criteria
     *
     * @param CollectionProcessor $collectionProcessor
     * @param callable $proceed
     * @param SearchCriteriaInterface $searchCriteria
     * @param AbstractDb $collection
     *
     * @return void
     */
    public function aroundProcess(
        CollectionProcessor $collectionProcessor,
        callable $proceed,
        SearchCriteriaInterface $searchCriteria,
        AbstractDb $collection
    ) {
        // preserve original filter groups
        $originalFilterGroups = $searchCriteria->getFilterGroups();
        /**
         * @var $groupSearchCriteria \Magento\Framework\Api\SearchCriteriaBuilder[]
         */
        $groupSearchCriteria = [];

        if ($collection instanceof Collection) {
            $modifiedFilterGroups = $searchCriteria->getFilterGroups();

            foreach ($modifiedFilterGroups as $key => $filterGroup) {
                foreach ($this->filterGroupProcessors as $groupName => $groupProcessor) {
                    if ($groupProcessor->handles($filterGroup)) {
                        if (! isset($groupSearchCriteria[$groupName])) {
                            $groupSearchCriteria[$groupName] = $this->searchCriteriaBuilderFactory->create();
                        }

                        $groupSearchCriteria[$groupName]->addFilters($filterGroup->getFilters());
                        unset($modifiedFilterGroups[$key]);
                    }
                }
            }

            if (! empty($groupSearchCriteria)) {
                $searchCriteria->setFilterGroups($modifiedFilterGroups);
            }
        }

        $result = $proceed($searchCriteria, $collection);

        if (! empty($groupSearchCriteria)) {
            foreach ($groupSearchCriteria as $groupName => $searchCriteriaBuilder) {
                $this->filterGroupProcessors[$groupName]->apply(
                    $collection,
                    $searchCriteriaBuilder->create()
                );
            }

            // assign preserved filter groups
            $searchCriteria->setFilterGroups($originalFilterGroups);
        }

        return $result;
    }
}
