<?php

namespace SyncSpider\APIextension\Plugin\Magento\UrlRewrite\Model;

use Magento\Framework\Exception\AlreadyExistsException;
use Magento\UrlRewrite\Model\UrlPersistInterface;
use Magento\UrlRewrite\Service\V1\Data\UrlRewrite;
use Magento\Catalog\Api\ProductRepositoryInterface;
use Magento\Catalog\Model\Product\Action;
use Magento\CatalogUrlRewrite\Model\ProductUrlPathGenerator;
use Magento\CatalogUrlRewrite\Model\ProductUrlRewriteGenerator;
use Magento\UrlRewrite\Model\UrlFinderInterface;
use Magento\Framework\App\Config\ScopeConfigInterface;

class UrlPersistInterfacePlugin
{
    public const HANDLE_PRODUCT_URL_COLLUSION = 'syncspider/general/handle_product_url_collusion';

    /**
     * @var ProductRepositoryInterface
     */
    private $productRepository;

    /**
     * @var ProductUrlPathGenerator
     */
    private $productUrlPathGenerator;

    /**
     * @var ProductUrlRewriteGenerator
     */
    private $productUrlRewriteGenerator;

    /**
     * @var Action
     */
    private $productAction;

    /**
     * @var UrlFinderInterface
     */
    private $urlFinder;

    /**
     * @var ScopeConfigInterface
     */
    private $scopeConfig;

    /**
     * UrlPersistInterfacePlugin constructor
     *
     * @param ProductRepositoryInterface $productRepository
     * @param ProductUrlPathGenerator $productUrlPathGenerator
     * @param ProductUrlRewriteGenerator $productUrlRewriteGenerator
     * @param Action $productAction
     * @param UrlFinderInterface $urlFinder
     * @param ScopeConfigInterface $scopeConfig
     */
    public function __construct(
        ProductRepositoryInterface $productRepository,
        ProductUrlPathGenerator $productUrlPathGenerator,
        ProductUrlRewriteGenerator $productUrlRewriteGenerator,
        Action $productAction,
        UrlFinderInterface $urlFinder,
        ScopeConfigInterface $scopeConfig
    ) {
        $this->productRepository = $productRepository;
        $this->productUrlPathGenerator = $productUrlPathGenerator;
        $this->productUrlRewriteGenerator = $productUrlRewriteGenerator;
        $this->productAction = $productAction;
        $this->urlFinder = $urlFinder;
        $this->scopeConfig = $scopeConfig;
    }

    /**
     * Whether the product url colussion should be handled
     *
     * @return bool
     */
    public function handleProductUrlCollusion()
    {
        return !! $this->scopeConfig->getValue(static::HANDLE_PRODUCT_URL_COLLUSION);
    }

    /**
     * Whether given url is product entity url rewrite
     *
     * @param UrlRewrite $url
     *
     * @return bool
     */
    public function isProductUrl($url): bool
    {
        return $url && $url instanceof UrlRewrite && $url->getEntityType() === 'product';
    }

    /**
     * Whether given list of urls are product entity url rewrites and for the same product entity
     *
     * @param UrlRewrite[] $urls
     *
     * @return bool
     */
    public function areProductUrls(array $urls): bool
    {
        $baseUrl = reset($urls);

        if ($this->isProductUrl($baseUrl)) {
            foreach ($urls as $url) {
                if (! $this->isProductUrl($url) || $url->getEntityId() !== $baseUrl->getEntityId()) {
                    return false;
                }
            }

            return true;
        }

        return false;
    }

    /**
     * Whether any given product entity url rewrites aready exists
     *
     * @param UrlRewrite[] $urls
     *
     * @return bool
     */
    public function hasProductUrlCollusion(array $urls): bool
    {
        if ($this->areProductUrls($urls)) {
            $baseUrl = reset($urls);
            $data = [
                UrlRewrite::REQUEST_PATH => []
            ];

            foreach ($urls as $url) {
                $data[UrlRewrite::REQUEST_PATH][] = $url->getRequestPath();
            }

            $data[UrlRewrite::REQUEST_PATH] = array_unique($data[UrlRewrite::REQUEST_PATH]);
            $existingUrls = $this->urlFinder->findAllByData($data);

            foreach ($existingUrls as $key => $url) {
                if (! $this->isProductUrl($url) || $url->getEntityId() !== $baseUrl->getEntityId()) {
                    return true;
                }
            }
        }

        return false;
    }

    /**
     * In case the given product entity url rewrites have collusions generate new url key and rewrites
     *
     * The current url key is suffixed with product id and new url rewrites are generated based the new suffixed url key
     * to prevent url key collusions when saving the product url rewrites
     *
     * @param UrlPersistInterface $subject
     * @param callable $proceed
     * @param array $urls
     * @return mixed
     *
     * @throws AlreadyExistsException
     */
    public function aroundReplace(UrlPersistInterface $subject, callable $proceed, array $urls)
    {
        if ($this->handleProductUrlCollusion() && $this->hasProductUrlCollusion($urls)) {
            $url = reset($urls);
            $product = $this->productRepository->getById($url->getEntityId());
            $product->setUrlKey($product->getUrlKey() . '-' . $product->getId());
            $product->setUrlPath($this->productUrlPathGenerator->getUrlPath($product));
            $result = $proceed($this->productUrlRewriteGenerator->generate($product));
            $this->productAction->updateAttributes([$product->getId()], [
                'url_key' => $product->getUrlKey(),
                // 'url_path' => $product->getUrlPath() seems that url_path attribute is not saved in db so we skip it
            ], $product->getStoreId());
        } else {
            $result = $proceed($urls);
        }

        return $result;
    }
}
