<?php

namespace SyncSpider\APIextension\Plugin\Magento\Webapi\Controller;

use Magento\Framework\Filesystem;
use Magento\Framework\App\RequestInterface;
use Magento\Framework\App\Config\ScopeConfigInterface;
use Magento\Framework\App\Filesystem\DirectoryList;
use Magento\Webapi\Controller\Rest;
use Exception;

class RestPlugin
{
    public const ENABLE_API_REQUEST_LOGGING = 'syncspider/general/enable_api_request_logging';

    public const LOG_DIRECTORY = 'syncspider_debug';

    /**
     * @var Filesystem
     */
    private $filesystem;

    /**
     * @var ScopeConfigInterface
     */
    private $scopeConfig;

    /**
     * RestPlugin constructor.
     *
     * @param Filesystem $filesystem
     * @param ScopeConfigInterface $scopeConfig
     */
    public function __construct(Filesystem $filesystem, ScopeConfigInterface $scopeConfig)
    {
        $this->filesystem = $filesystem;
        $this->scopeConfig = $scopeConfig;
    }
    
    /**
     * Whether enable api request logging
     *
     * @return boolean
     */
    public function enableApiRequestLogging(): bool
    {
        return !!$this->scopeConfig->getValue(static::ENABLE_API_REQUEST_LOGGING);
    }

    /**
     * Get api request log directory
     *
     * @return string
     */
    public function getApiRequestLogDirectory(): string
    {
        $read = $this->filesystem->getDirectoryRead(DirectoryList::LOG);

        if (! $read->isExist(static::LOG_DIRECTORY)) {
            $write = $this->filesystem->getDirectoryWrite(DirectoryList::LOG);
            $write->create(static::LOG_DIRECTORY);
        }

        return $read->getAbsolutePath(static::LOG_DIRECTORY);
    }

    /**
     * Whether given request interface supports required methods for logging
     *
     * @param RequestInterface $request
     *
     * @return boolean
     */
    public function supportsLogging(RequestInterface $request): bool
    {
        $methodList = [
            'getMethod',
            'getPathInfo',
            'getRequestString',
            'getContent'
        ];

        foreach ($methodList as $method) {
            if (! method_exists($request, $method)) {
                return false;
            }
        }

        return true;
    }

    /**
     * Log incoming API requests for debug purposes
     *
     * @param Rest $subject
     * @param RequestInterface $request
     *
     * @return void
     */
    public function beforeDispatch(Rest $subject, RequestInterface $request)
    {
        try {
            if ($this->enableApiRequestLogging() && $this->supportsLogging($request)) {
                $file = $this->getApiRequestLogDirectory() . '/request-' . time();
                $suffix = '';
                $counter = 0;

                while (file_exists($file . $suffix)) {
                        $suffix = '_' . $counter++;
                }

                file_put_contents($file . $suffix, [
                    $request->getMethod(),
                    $request->getPathInfo(),
                    "\n",
                    $request->getRequestString(),
                    "\n",
                    $request->getContent()
                ]);
            }
        } catch (Exception $e) {
            // suppress exceptions to prevent distruption in API
        }
    }
}
