<?php

namespace SyncSpider\APIextension\Shipping\Carrier;

use Magento\Framework\App\ObjectManager;
use SyncSpider\APIextension\Api\Data\OrderShippingInterface;

class SyncSpider extends \Magento\Shipping\Model\Carrier\AbstractCarrier implements
    \Magento\Shipping\Model\Carrier\CarrierInterface
{
    /**
     * Carrier code
     *
     * @var string
     */
    protected $_code = \SyncSpider\APIextension\Api\CarrierInterface::CODE;

    /**
     * @var \Magento\Shipping\Model\Rate\ResultFactory
     */
    private $rateResultFactory;

    /**
     * @var \Magento\Quote\Model\Quote\Address\RateResult\MethodFactory
     */
    private $rateMethodFactory;

    /**
     * @param \Magento\Framework\App\Config\ScopeConfigInterface $scopeConfig
     * @param \Magento\Quote\Model\Quote\Address\RateResult\ErrorFactory $rateErrorFactory
     * @param \Psr\Log\LoggerInterface $logger
     * @param \Magento\Shipping\Model\Rate\ResultFactory $rateResultFactory
     * @param \Magento\Quote\Model\Quote\Address\RateResult\MethodFactory $rateMethodFactory
     * @param array $data
     */
    public function __construct(
        \Magento\Framework\App\Config\ScopeConfigInterface $scopeConfig,
        \Magento\Quote\Model\Quote\Address\RateResult\ErrorFactory $rateErrorFactory,
        \Psr\Log\LoggerInterface $logger,
        \Magento\Shipping\Model\Rate\ResultFactory $rateResultFactory,
        \Magento\Quote\Model\Quote\Address\RateResult\MethodFactory $rateMethodFactory,
        array $data = []
    ) {
        $this->rateResultFactory = $rateResultFactory;
        $this->rateMethodFactory = $rateMethodFactory;
        parent::__construct($scopeConfig, $rateErrorFactory, $logger, $data);
    }

    /**
     * @inheritdoc
     */
    public function collectRates(\Magento\Quote\Model\Quote\Address\RateRequest $request)
    {
        /** @var \Magento\Shipping\Model\Rate\Result $result */
        $result = $this->rateResultFactory->create();

        /** @var \SyncSpider\APIextension\Shipping\Carrier\TransferObject $transferObject */
        $transferObject = ObjectManager::getInstance()->get(TransferObject::class);

        if (!$transferObject->isUsed()) {
            return false;
        }

        $shipping = $transferObject->getShipping();
        $shippingCost = 0;

        /** @var \Magento\Quote\Model\Quote\Address\RateResult\Method $method */
        $method = $this->rateMethodFactory->create();
        $method->setMethod($this->_code);
        $method->setCarrier($this->_code);

        if ($shipping instanceof OrderShippingInterface) {
            $shippingCost = $shipping->getShippingCost() === null ? $shippingCost : $shipping->getShippingCost();
            $method->setCarrierTitle($shipping->getCarrierTitle() ? $shipping->getCarrierTitle() : __('Generic'));
            $method->setMethodTitle(implode(' | ', $this->getParts($shipping)));
        }

        $method->setPrice($shippingCost);
        $method->setCost($shippingCost);
        $result->append($method);

        return $result;
    }

    /**
     * @inheritDoc
     */
    public function checkAvailableShipCountries(\Magento\Framework\DataObject $request)
    {
        /** @var \SyncSpider\APIextension\Shipping\Carrier\TransferObject $transferObject */
        $transferObject = ObjectManager::getInstance()->get(TransferObject::class);
        return $transferObject->isUsed();
    }

    /**
     * @inheritdoc
     */
    public function getAllowedMethods()
    {
        return ['generic' => __('Generic')];
    }

    /**
     * Check if carrier has shipping tracking option available
     *
     * @return boolean
     */
    public function isTrackingAvailable()
    {
        return false;
    }

    /**
     * Get shipping parts based on given order shipping
     *
     * @param \SyncSpider\APIextension\Api\Data\OrderShippingInterface $shipping
     *
     * @return array
     */
    private function getParts($shipping)
    {
        $result = [];

        if ($shipping->getCarrierCode()) {
            $result[] = __('Carrier Code: %1', $shipping->getCarrierCode());
        }

        if ($shipping->getMethodTitle()) {
            $result[] = __('Method Title: %1', $shipping->getShippingMethodTitle());
        }

        if ($shipping->getMinDeliveryDate()) {
            $result[] = __('Estimated Delivery Date From: %1', $shipping->getMinDeliveryDate());
        }

        if ($shipping->getMaxDeliveryDate()) {
            $result[] = __('Estimated Delivery Date To: %1', $shipping->getMaxDeliveryDate());
        }

        foreach ($shipping->getShippingAdditional() as $additional) {
            $result[] = __("%1: %2", $additional->getKey(), $additional->getValue());
        }

        return $result;
    }
}
