<?php

namespace SyncSpider\APIextension\ViewModel\Order;

use Magento\Framework\Api\SearchCriteriaBuilderFactory;
use Magento\Framework\View\LayoutInterface;
use Magento\Framework\Stdlib\DateTime\TimezoneInterface;
use Magento\Sales\Api\{
    Data\OrderInterface,
    OrderRepositoryInterface
};
use SyncSpider\APIextension\Api\Data\SynchronizationDataInterface;
use SyncSpider\APIextension\ViewModel\AbstractViewModel;
use IntlDateFormatter;
use Exception;

class ExtensionAttributes extends AbstractViewModel
{
    /**
     * @var LayoutInterface
     */
    private $layout;

    /**
     * @var OrderRepositoryInterface
     */
    private $orderRepository;

    /**
     * @var SearchCriteriaBuilderFactory
     */
    private $searchCriteriaBuilderFactory;

    /**
     * @var TimezoneInterface
     */
    private $timezone;

    /**
     * @var OrderInterface[]
     */
    private $cache;

    /**
     * ExtensionAttributes view model constructor.
     *
     * @param LayoutInterface $layout
     * @param OrderRepositoryInterface $orderRepository
     * @param SearchCriteriaBuilderFactory $searchCriteriaBuilderFactory
     * @param TimezoneInterface $timezone
     */
    public function __construct(
        LayoutInterface $layout,
        OrderRepositoryInterface $orderRepository,
        SearchCriteriaBuilderFactory $searchCriteriaBuilderFactory,
        TimezoneInterface $timezone
    ) {
        $this->layout = $layout;
        $this->orderRepository = $orderRepository;
        $this->searchCriteriaBuilderFactory = $searchCriteriaBuilderFactory;
        $this->timezone = $timezone;
    }

    /**
     * Attach extension attributes to given order
     *
     * @param OrderInterface $order
     *
     * @return OrderInterface
     */
    public function attachExtensionAttributes(OrderInterface $order): OrderInterface
    {
        if (! isset($this->cache[$order->getEntityId()])) {
            $this->cache[$order->getEntityId()] = $order;
            $searchCriteria = $this->searchCriteriaBuilderFactory->create()
                ->addFilter('entity_id', $order->getEntityId())
                ->setPageSize(1)
                ->create();
            $result = $this->orderRepository->getList($searchCriteria);
            $items = $result->getItems();

            if (count($items) === 1) {
                $this->cache[$order->getEntityId()] = array_pop($items);
            }
        }

        return $this->cache[$order->getEntityId()] ?: $order;
    }

    /**
     * Get current order from layout "order_info" block
     *
     * @return OrderInterface|null
     */
    public function getOrder(): ?OrderInterface
    {
        try {
            $orderSourceBlock = $this->layout->getBlock('order_info');

            if ($orderSourceBlock && method_exists($orderSourceBlock, 'getOrder')) {
                $order = $orderSourceBlock->getOrder();

                if ($order instanceof OrderInterface) {
                    return $this->attachExtensionAttributes($order);
                }
            }
        } catch (Exception $e) {
            // suppress exceptions
        }

        return null;
    }

    /**
     * Render syncspider read date
     *
     * @param OrderInterface $order
     *
     * @return string
     */
    public function renderSyncspiderReadDate(OrderInterface $order)
    {
        $html = '';

        if ($order->getExtensionAttributes()) {
            $html = $this->formatDateTime(
                $order->getExtensionAttributes()->getSyncspiderReadDate()
            );

            if ($order->getExtensionAttributes()->getSyncspiderSynchronizationData()) {
                $html = $this->formatDateTime(
                    $order->getExtensionAttributes()->getSyncspiderSynchronizationData()->getTimestamp()
                );
                $link = $this->getSynchronizationDataLink(
                    $order->getExtensionAttributes()->getSyncspiderSynchronizationData()
                );
                $html = "<a href=\"{$link}\" target=\"_blank\" rel=\"noreferrer nofollow\">{$html}</a>";
            }
        }

        return $html;
    }

    /**
     * Format given timestamp as common date time
     *
     * @param string|null $timestamp
     *
     * @return string
     */
    private function formatDateTime(string $timestamp = null): string
    {
        if (empty($timestamp)) {
            return '';
        }

        return $this->timezone->formatDateTime(
            $timestamp,
            IntlDateFormatter::MEDIUM,
            IntlDateFormatter::MEDIUM
        );
    }

    /**
     * Get synnchronization data link
     *
     * @param SynchronizationDataInterface $synchronizationData
     *
     * @return string
     */
    private function getSynchronizationDataLink(SynchronizationDataInterface $synchronizationData): string
    {
        return implode('/', [
            'https://app.syncspider.com/app/project',
            $synchronizationData->getProjectId(),
            'task',
            $synchronizationData->getTaskId(),
            'execution',
            $synchronizationData->getExecutionId()
        ]);
    }
}
